@@ -4,31 +4,30 @@
 """RAMSES RF - a RAMSES-II protocol decoder & analyser."""
 
 import logging
-from inspect import getmembers, isclass
 from random import randint
-from sys import modules
 from typing import Dict, Optional
 
-from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, DHW_HACK, Discover, __dev_mode__
-from .entities import Entity, discover_decorator
-from .protocol import Command, Priority  # TODO: constants to const.py
-from .protocol.address import NON_DEV_ADDR, id_to_address
-from .protocol.command import FUNC, TIMEOUT
-from .protocol.const import (
+from .const import (
     _000C_DEVICE,
+    _0005_ZONE,
     ATTR_HEAT_DEMAND,
     ATTR_RELAY_DEMAND,
     ATTR_SETPOINT,
     ATTR_TEMP,
     ATTR_WINDOW_OPEN,
     BOOST_TIMER,
-    DEVICE_CLASS,
+    DEV_KLASS,
     DEVICE_TYPES,
     DOMAIN_TYPE_MAP,
     FAN_MODE,
     NUL_DEVICE_ID,
+    Discover,
+    __dev_mode__,
 )
-from .protocol.exceptions import CorruptStateError
+from .entities import Entity, class_by_attr, discover_decorator
+from .protocol import Command, CorruptStateError, Priority
+from .protocol.address import NON_DEV_ADDR, id_to_address
+from .protocol.command import FUNC, TIMEOUT
 from .protocol.opentherm import (
     MSG_ID,
     MSG_NAME,
@@ -41,6 +40,7 @@ from .protocol.opentherm import (
 )
 from .protocol.ramses import CODE_ONLY_FROM_CTL, RAMSES_DEVICES
 from .protocol.transport import PacketProtocolPort
+from .schema import SZ_ALIAS, SZ_CLASS, SZ_DEVICE_ID, SZ_FAKED
 
 from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
@@ -118,38 +118,38 @@ _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
-_DEV_TYPE_TO_CLASS = {  # TODO: *remove*
-    None: DEVICE_CLASS.GEN,  # a generic, promotable device
-    "00": DEVICE_CLASS.TRV,
-    "01": DEVICE_CLASS.CTL,
-    "02": DEVICE_CLASS.UFC,
-    "03": DEVICE_CLASS.STA,
-    "04": DEVICE_CLASS.TRV,
-    "07": DEVICE_CLASS.DHW,
-    "10": DEVICE_CLASS.OTB,
-    "12": DEVICE_CLASS.STA,  # 12: can act like a DEVICE_CLASS.PRG
-    "13": DEVICE_CLASS.BDR,
-    "17": DEVICE_CLASS.EXT,
-    "18": DEVICE_CLASS.HGI,
-    "20": DEVICE_CLASS.FAN,
-    "22": DEVICE_CLASS.STA,  # 22: can act like a DEVICE_CLASS.PRG
-    "23": DEVICE_CLASS.PRG,
-    "29": DEVICE_CLASS.FAN,
-    "30": DEVICE_CLASS.GEN,  # either: RFG/FAN
-    "32": DEVICE_CLASS.HUM,  # also: SWI
-    "34": DEVICE_CLASS.STA,
-    "37": DEVICE_CLASS.FAN,
-    "39": DEVICE_CLASS.SWI,
-    "42": DEVICE_CLASS.SWI,
-    "49": DEVICE_CLASS.SWI,
-    "59": DEVICE_CLASS.SWI,
+_DEV_TYPE_TO_KLASS = {  # TODO: *remove*
+    None: DEV_KLASS.GEN,  # a generic, promotable device
+    "00": DEV_KLASS.TRV,
+    "01": DEV_KLASS.CTL,
+    "02": DEV_KLASS.UFC,
+    "03": DEV_KLASS.STA,
+    "04": DEV_KLASS.TRV,
+    "07": DEV_KLASS.DHW,
+    "10": DEV_KLASS.OTB,
+    "12": DEV_KLASS.STA,  # 12: can act like a DEV_KLASS.PRG
+    "13": DEV_KLASS.BDR,
+    "17": DEV_KLASS.EXT,
+    "18": DEV_KLASS.HGI,
+    "20": DEV_KLASS.FAN,
+    "22": DEV_KLASS.STA,  # 22: can act like a DEV_KLASS.PRG
+    "23": DEV_KLASS.PRG,
+    "29": DEV_KLASS.FAN,
+    "30": DEV_KLASS.GEN,  # either: RFG/FAN
+    "32": DEV_KLASS.HUM,  # also: SWI
+    "34": DEV_KLASS.STA,
+    "37": DEV_KLASS.FAN,
+    "39": DEV_KLASS.SWI,
+    "42": DEV_KLASS.SWI,
+    "49": DEV_KLASS.SWI,
+    "59": DEV_KLASS.SWI,
 }  # these are the default device classes for common types
 
 
 class DeviceBase(Entity):
     """The Device base class (good for a generic device)."""
 
-    _class = None
+    _klass = None
     _types = tuple()  # TODO: needed?
 
     def __init__(self, gwy, dev_addr, ctl=None, domain_id=None, **kwargs) -> None:
@@ -178,7 +178,7 @@ class DeviceBase(Entity):
         self._alias = None
         self._faked = None
         if self.id in gwy._include:
-            self._alias = gwy._include[self.id].get(ATTR_ALIAS)
+            self._alias = gwy._include[self.id].get(SZ_ALIAS)
 
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id})"
@@ -241,8 +241,6 @@ class DeviceBase(Entity):
         return ctl
 
     def _handle_msg(self, msg) -> None:
-        if DHW_HACK and msg.code == _1260:
-            _LOGGER.debug(f"{msg._pkt} < handling (11)")  # HACK: lloyda
         assert msg.src is self, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
@@ -273,6 +271,11 @@ class DeviceBase(Entity):
 
         return False
 
+    # @property
+    # def _is_parent(self) -> bool:
+    #     """Return True if other devices can bind to this device."""
+    #     return self._klass in (DEV_KLASS.CTL, DEV_KLASS.PRG, DEV_KLASS.UFC)
+
     @property
     def _is_present(self) -> bool:
         """Try to exclude ghost devices (as caused by corrupt packet addresses)."""
@@ -294,9 +297,9 @@ class DeviceBase(Entity):
 
         return {
             **(self._codes if DEV_MODE else {}),
-            ATTR_ALIAS: self._alias,
-            # ATTR_FAKED: self._faked,
-            ATTR_CLASS: self._class,
+            SZ_ALIAS: self._alias,
+            # SZ_FAKED: self._faked,
+            SZ_CLASS: self._klass,
         }
 
     @property
@@ -339,12 +342,10 @@ class DeviceInfo:  # 10E0
 class Device(DeviceInfo, DeviceBase):
     """The Device base class - also used for unknown device types."""
 
-    _class = DEVICE_CLASS.GEN
+    _klass = DEV_KLASS.GEN
     DEVICE_TYPES = tuple()
 
     def _handle_msg(self, msg) -> None:
-        if DHW_HACK and msg.code == _1260:
-            _LOGGER.debug(f"{msg._pkt} < handling (10)")  # HACK: lloyda
         super()._handle_msg(msg)
 
         if type(self) is Device and self.type == "30":  # self.__class__ is Device, DEX
@@ -430,13 +431,6 @@ class Actuator:  # 3EF0, 3EF1
     ACTUATOR_STATE = "actuator_state"
     MODULATION_LEVEL = "modulation_level"  # percentage (0.0-1.0)
 
-    def _discover(self, discover_flag=Discover.ALL) -> None:
-        super()._discover(discover_flag=discover_flag)
-
-        if discover_flag & Discover.STATUS and not self._faked:
-            # NOTE: No need to send periodic RQ/3EF1s to an OTB, use RQ/3220/11s
-            self._make_cmd(_3EF1)  # NOTE: No RPs to RQ/3EF0
-
     def _handle_msg(self, msg) -> None:  # NOTE: active
         super()._handle_msg(msg)
 
@@ -451,6 +445,32 @@ class Actuator:  # 3EF0, 3EF1
         ):
             self._make_cmd(_3EF1, priority=Priority.LOW, retries=1)
 
+    @property
+    def _ch_active(self) -> Optional[bool]:  # 3EF0
+        return self._msg_value(_3EF0, key="ch_active")
+
+    @property
+    def _ch_enabled(self) -> Optional[bool]:  # 3EF0
+        return self._msg_value(_3EF0, key="ch_enabled")
+
+    @property
+    def _dhw_active(self) -> Optional[bool]:  # 3EF0
+        return self._msg_value(_3EF0, key="dhw_active")
+
+    @property
+    def _flame_active(self) -> Optional[bool]:  # 3EF0
+        return self._msg_value(_3EF0, key="flame_active")
+
+    @property
+    def _bit_3_7(self) -> Optional[bool]:  # 3EF0
+        if flags := self._msg_value(_3EF0, key="_flags_3"):
+            return flags[7]
+
+    @property
+    def _bit_6_6(self) -> Optional[bool]:  # 3EF0 ?dhw_enabled
+        if flags := self._msg_value(_3EF0, key="_flags_6"):
+            return flags[6]
+
     @property
     def actuator_cycle(self) -> Optional[dict]:  # 3EF1
         return self._msg_value(_3EF1)
@@ -529,7 +549,7 @@ class Fakeable:
     def __init__(self, gwy, *args, **kwargs) -> None:
         super().__init__(gwy, *args, **kwargs)
 
-        if self.id in gwy._include and gwy._include[self.id].get(ATTR_FAKED):
+        if self.id in gwy._include and gwy._include[self.id].get(SZ_FAKED):
             self._make_fake()
 
     def _bind(self):
@@ -539,7 +559,7 @@ class Fakeable:
     def _make_fake(self, bind=None) -> Device:
         if not self._faked:
             self._faked = True
-            self._gwy._include[self.id] = {ATTR_FAKED: True}
+            self._gwy._include[self.id] = {SZ_FAKED: True}
             _LOGGER.warning(f"Faking now enabled for {self}")  # TODO: be info/debug
         if bind:
             self._bind()
@@ -632,7 +652,7 @@ class Fakeable:
     def schema(self) -> dict:
         return {
             **super().schema,
-            ATTR_FAKED: self._faked,
+            SZ_FAKED: self._faked,
         }
 
 
@@ -643,7 +663,7 @@ class Weather(Fakeable):  # 0002 (fakeable)
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        if kwargs.get(ATTR_FAKED) is True or _0002 in kwargs.get(ATTR_FAKED, []):
+        if kwargs.get(SZ_FAKED) is True or _0002 in kwargs.get(SZ_FAKED, []):
             self._make_fake()
 
     def _bind(self):
@@ -685,7 +705,7 @@ class Temperature(Fakeable):  # 30C9 (fakeable)
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        if kwargs.get(ATTR_FAKED) is True or _30C9 in kwargs.get(ATTR_FAKED, []):
+        if kwargs.get(SZ_FAKED) is True or _30C9 in kwargs.get(SZ_FAKED, []):
             self._make_fake()
 
     def _bind(self):
@@ -728,7 +748,7 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        if kwargs.get(ATTR_FAKED) is True or _3EF0 in kwargs.get(ATTR_FAKED, []):
+        if kwargs.get(SZ_FAKED) is True or _3EF0 in kwargs.get(SZ_FAKED, []):
             self._make_fake()
 
     @discover_decorator
@@ -809,14 +829,14 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
 class RfgGateway(DeviceInfo, DeviceBase):  # RFG (30:)
     """The RFG100 base class."""
 
-    _class = DEVICE_CLASS.RFG
+    _klass = DEV_KLASS.RFG
     _types = ("30",)
 
 
 class HgiGateway(DeviceBase):  # HGI (18:), was GWY
     """The HGI80 base class."""
 
-    _class = DEVICE_CLASS.HGI
+    _klass = DEV_KLASS.HGI
     _types = ("18",)
 
     def __init__(self, *args, **kwargs) -> None:
@@ -941,7 +961,7 @@ class HgiGateway(DeviceBase):  # HGI (18:), was GWY
     @property
     def schema(self):
         return {
-            "device_id": self.id,
+            SZ_DEVICE_ID: self.id,
             "faked_bdr": self._faked_bdr and self._faked_bdr.id,
             "faked_ext": self._faked_ext and self._faked_ext.id,
             "faked_thm": self._faked_thm and self._faked_thm.id,
@@ -951,7 +971,7 @@ class HgiGateway(DeviceBase):  # HGI (18:), was GWY
 class Controller(Device):  # CTL (01):
     """The Controller base class."""
 
-    _class = DEVICE_CLASS.CTL
+    _klass = DEV_KLASS.CTL
     _types = ("01",)
 
     def __init__(self, *args, **kwargs) -> None:
@@ -987,14 +1007,14 @@ class Controller(Device):  # CTL (01):
 class Programmer(Controller):  # PRG (23):
     """The Controller base class."""
 
-    _class = DEVICE_CLASS.PRG
+    _klass = DEV_KLASS.PRG
     _types = ("23",)
 
 
 class UfhController(Device):  # UFC (02):
     """The UFC class, the HCE80 that controls the UFH zones."""
 
-    _class = DEVICE_CLASS.UFC
+    _klass = DEV_KLASS.UFC
     _types = ("02",)
 
     HEAT_DEMAND = ATTR_HEAT_DEMAND
@@ -1018,19 +1038,19 @@ class UfhController(Device):  # UFC (02):
     def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        # TODO: UFC may RP to an RQ/0001
-
         if discover_flag & Discover.SCHEMA:
-            [  # 0005: shows which channels are active - ?no use? (see above)
-                self._make_cmd(_0005, payload=f"00{zone_type}")
-                for zone_type in ("00", "04", "09", "0F")  # TODO: are all req'd
-                # for zone_type in _0005_ZONE_TYPE
-            ]
+            for zone_type in (_0005_ZONE.ALL, _0005_ZONE.ALL_SENSOR, _0005_ZONE.UFH):
+                # TODO: are all three needed?
+                try:  # 0005: shows which channels are active - ?no use? (see above)
+                    _ = self._msgz[_0005][RP][f"00{zone_type}"]
+                except KeyError:
+                    self._make_cmd(_0005, payload=f"00{zone_type}")
 
-            [  # 000C: used to find evo zone for each configured channel
-                self._make_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.ALL}")
-                for idx in range(8)  # for each possible UFH channel/circuit
-            ]  # _000C_DEVICE.UFH doesn't seem to work with all UFCs
+            for idx in range(8):
+                try:  # _000C_DEVICE.UFH doesn't seem to work with all UFCs??
+                    _ = self._msgz[_000C][RP][f"{idx:02X}{_000C_DEVICE.ALL}"]
+                except KeyError:
+                    self._make_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.ALL}")
 
         # if discover_flag & Discover.STATUS:
         #     [  # 22C9: no answer
@@ -1147,7 +1167,7 @@ class UfhController(Device):  # UFC (02):
 class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
     """The DHW class, such as a CS92."""
 
-    _class = DEVICE_CLASS.DHW
+    _klass = DEV_KLASS.DHW
     _types = ("07",)
 
     DHW_PARAMS = "dhw_params"
@@ -1163,16 +1183,8 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
         return f"{self.id} ({self._domain_id}): {self.temperature}"
 
     def _handle_msg(self, msg) -> None:  # NOTE: active
-        if DHW_HACK:
-            _LOGGER.debug(f"{msg._pkt} < handling (00)")  # HACK: lloyda
-            _LOGGER.debug(f"msgs = {self._msgs}")
-
         super()._handle_msg(msg)
 
-        if DHW_HACK:
-            _LOGGER.debug(f"{msg._pkt} < handled. (00)")  # HACK: lloyda
-            _LOGGER.debug(f"msgs = {self._msgs}")
-
         if msg.code == _1260 and self._ctl and not self._gwy.config.disable_sending:
             # device can be instantiated with a CTL
             self._send_cmd(Command.get_dhw_temp(self._ctl.id))
@@ -1199,15 +1211,6 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
         ) as exc:
             _LOGGER.exception(exc)
 
-        if DHW_HACK and result is None:  # HACK: lloyda
-            msg = self._msgs.get(_1260)
-            _LOGGER.info(f"DHW msg 1 = {msg!r}")
-            if msg:
-                result = msg.payload.get(self.TEMPERATURE)
-            else:
-                _LOGGER.info(f"DHW msg 2 = {msg!r}")
-                _LOGGER.debug(f"msgs = {self._msgs}")
-                _LOGGER.debug(f"msgz = {self._msgz}")
         return result
 
     @property
@@ -1228,7 +1231,7 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
 class ExtSensor(Weather, Device):  # EXT: 17
     """The EXT class (external sensor), such as a HB85/HB95."""
 
-    _class = DEVICE_CLASS.EXT
+    _klass = DEV_KLASS.EXT
     _types = ("17",)
 
     # LUMINOSITY = "luminosity"  # lux
@@ -1244,7 +1247,7 @@ class ExtSensor(Weather, Device):  # EXT: 17
 class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     """The OTB class, specifically an OpenTherm Bridge (R8810A Bridge)."""
 
-    _class = DEVICE_CLASS.OTB
+    _klass = DEV_KLASS.OTB
     _types = ("10",)
 
     # BOILER_SETPOINT = "boiler_setpoint"
@@ -1263,6 +1266,20 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.rel_modulation_level}"  # 3EF0
 
+    def _start_discovery(self) -> None:
+
+        delay = randint(10, 20)
+
+        self._gwy._add_task(  # 10E0/1FC9, 3220 pkts
+            self._discover, discover_flag=Discover.SCHEMA, delay=0, period=3600 * 24
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.PARAMS, delay=delay, period=3600
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.STATUS, delay=delay + 1, period=60
+        )
+
     @discover_decorator
     def _discover(self, discover_flag=Discover.ALL) -> None:
         # see: https://www.opentherm.eu/request-details/?post_ids=2944
@@ -1300,6 +1317,8 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
                 self._send_cmd(Command(RQ, code, "00", self.id))
 
         if discover_flag & Discover.STATUS:
+            # NOTE: No need to send periodic RQ/3EF1s to an OTB, use RQ/3220/11s
+            self._send_cmd(Command(RQ, _3EF0, "00", self.id))  # CTLs dont RP to RQ/3EF0
             self._send_cmd(Command(RQ, _12F0, "00", self.id))
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
@@ -1338,6 +1357,14 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
                 "-reserved-",
             )
 
+    @staticmethod
+    def _ot_msg_name(msg) -> str:
+        return (
+            msg.payload[MSG_NAME]
+            if isinstance(msg.payload[MSG_NAME], str)
+            else f"{msg.payload[MSG_ID]:02X}"
+        )
+
     def _ot_msg_value(self, msg_id) -> Optional[float]:
         if (
             (msg := self._opentherm_msg.get(msg_id))
@@ -1356,17 +1383,32 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
 
     @property
     def boiler_setpoint(self) -> Optional[float]:  # 3220/01 (22D9)
-        # return self._msg_value(_22D9, key=self.BOILER_SETPOINT)
         return self._ot_msg_value("01")
 
+    @property
+    def _boiler_setpoint(self) -> Optional[float]:  # 3220/01 (22D9)
+        return self._msg_value(_22D9, key="setpoint")
+
     @property
     def ch_max_setpoint(self) -> Optional[float]:  # 3220/39 (1081)
         return self._ot_msg_value("39")
 
+    @property
+    def _ch_max_setpoint(self) -> Optional[float]:  # 3220/39 (1081)
+        return self._msg_value(_1081, key="setpoint")
+
     @property
     def ch_water_pressure(self) -> Optional[float]:  # 3220/12 (1300)
         return self._ot_msg_value("12")
 
+    @property
+    def _ch_water_pressure(self) -> Optional[float]:  # 3220/12 (1300)
+        return self._msg_value(_1300, key="pressure")
+
+    @property
+    def _ch_setpoint(self) -> Optional[bool]:  # 3EF0
+        return self._msg_value(_3EF0, key="ch_setpoint")
+
     @property
     def dhw_flow_rate(self) -> Optional[float]:  # 3220/13
         return self._ot_msg_value("13")
@@ -1375,31 +1417,83 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     def dhw_setpoint(self) -> Optional[float]:  # 3220/38 (10A0)
         return self._ot_msg_value("38")
 
+    @property
+    def _dhw_setpoint(self) -> Optional[float]:  # 3220/38 (10A0)
+        return self._msg_value(_1300, key="setpoint")
+
     @property
     def dhw_temp(self) -> Optional[float]:  # 3220/1A (1260)
         return self._ot_msg_value("1A")
 
+    @property
+    def _dhw_temp(self) -> Optional[float]:  # 3220/1A (1260)
+        return self._msg_value(_1260, key="temperature")
+
     @property
     def outside_temp(self) -> Optional[float]:  # 3220/1B (1290)
         return self._ot_msg_value("1B")
 
     @property
-    def rel_modulation_level(self) -> Optional[float]:  # 3220/11 (3EFx)
-        # return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
-        return self._ot_msg_value("11")
-
-    @staticmethod
-    def _msg_name(msg) -> str:
-        return (
-            msg.payload[MSG_NAME]
-            if isinstance(msg.payload[MSG_NAME], str)
-            else f"{msg.payload[MSG_ID]:02X}"
-        )
+    def _outside_temp(self) -> Optional[float]:  # 3220/1B (1290)
+        return self._msg_value(_1290, key="temperature")
 
     @property
+    def rel_modulation_level(self) -> Optional[float]:  # 3220/11 (3EFx)
+        """Return the relative modulation level from OpenTherm."""
+        return self._ot_msg_value("11")
+
+    @property
+    def _rel_modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
+        """Return the relative modulation level from RAMSES_II."""
+        return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
+
+    @property
+    def _max_rel_modulation(self) -> Optional[float]:  # 3EF0
+        return self._msg_value(_3EF0, key="max_rel_modulation")
+
+    @property
+    def ch_active(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[8 + 1]
+
+    @property
+    def ch_enabled(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[0]
+
+    @property
+    def dhw_active(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[8 + 2]
+
+    @property
+    def dhw_enabled(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[1]
+
+    @property
+    def flame_active(self) -> Optional[bool]:  # 3220/00 (flame_on)
+        if flags := self._ot_msg_value("00"):
+            return flags[8 + 3]
+
+    @property
+    def cooling_active(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[8 + 4]
+
+    @property
+    def cooling_enabled(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[2]
+
+    @property
+    def fault_present(self) -> Optional[bool]:  # 3220/00
+        if flags := self._ot_msg_value("00"):
+            return flags[8]
+
     def opentherm_schema(self) -> dict:
         result = {
-            self._msg_name(v): v.payload
+            self._ot_msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
             if self._supported_msg.get(int(k, 16)) and int(k, 16) in SCHEMA_MSG_IDS
         }
@@ -1411,7 +1505,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     @property
     def opentherm_params(self) -> dict:
         result = {
-            self._msg_name(v): v.payload
+            self._ot_msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
             if self._supported_msg.get(int(k, 16)) and int(k, 16) in PARAMS_MSG_IDS
         }
@@ -1434,13 +1528,29 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
             "outside_temp": self.outside_temp,
             "rel_modulation_level": self.rel_modulation_level,
         }
-        # return {
-        #     slugify(self._opentherm_msg[msg_id].payload[MSG_NAME]): (
-        #         self._opentherm_msg[msg_id].payload[VALUE]
-        #     )
-        #     for msg_id in (0x11, 0x12, 0x13, 0x19, 0x1A, 0x1C)
-        #     if msg_id in self._opentherm_msg
-        # }
+
+    @property
+    def ramses_params(self) -> dict:
+        return {
+            "ch_max_setpoint": self._ch_max_setpoint,
+            "dhw_setpoint": self._dhw_setpoint,
+            "max_rel_modulation": self._max_rel_modulation,
+        }
+
+    @property
+    def ramses_status(self) -> dict:
+        return {
+            "boiler_setpoint": self._boiler_setpoint,
+            "ch_setpoint": self._ch_setpoint,
+            "ch_water_pressure": self._ch_water_pressure,
+            "dhw_temp": self._dhw_temp,
+            "outside_temp": self._outside_temp,
+            "rel_modulation_level": self._rel_modulation_level,
+            "ch_active": self._ch_active,
+            "ch_enabled": self._ch_enabled,
+            "dhw_active": self._dhw_active,
+            "flame_active": self._flame_active,
+        }
 
     @property
     def schema(self) -> dict:
@@ -1465,15 +1575,16 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     @property
     def status(self) -> dict:
         return {
-            **super().status,
+            **super().status,  # incl. actuator_cycle, actuator_state
             "opentherm_status": self.opentherm_status,
+            "ramses_status": self.ramses_status,
         }
 
 
 class Thermostat(BatteryState, Setpoint, Temperature, Device):  # THM (..):
     """The THM/STA class, such as a TR87RF."""
 
-    _class = DEVICE_CLASS.STA
+    _klass = DEV_KLASS.STA
     _types = ("03", "12", "22", "34")
 
     # _STATE = super().TEMPERATURE
@@ -1535,7 +1646,7 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
     - x2 DHW thingys (F9/DHW, FA/DHW)
     """
 
-    _class = DEVICE_CLASS.BDR
+    _klass = DEV_KLASS.BDR
     _types = ("13",)
 
     ACTIVE = "active"
@@ -1573,6 +1684,10 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
         if discover_flag & Discover.PARAMS and not self._faked:
             self._send_cmd(Command.get_tpi_params(self.id))  # or: self._ctl.id
 
+        if discover_flag & Discover.STATUS and not self._faked:
+            # NOTE: 13: wont RP to an RQ/3EF0
+            self._make_cmd(_3EF1)
+
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
@@ -1634,7 +1749,7 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
 class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # TRV (04):
     """The TRV class, such as a HR92."""
 
-    _class = DEVICE_CLASS.TRV
+    _klass = DEV_KLASS.TRV
     _types = ("00", "04")  # TODO: keep 00?
 
     WINDOW_OPEN = ATTR_WINDOW_OPEN  # boolean
@@ -1643,6 +1758,13 @@ class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # T
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.heat_demand}"
 
+    @property
+    def heat_demand(self) -> Optional[float]:  # 3150
+        if (heat_demand := super().heat_demand) is None:
+            if self._msg_value(_3150) is None and self.setpoint is False:
+                return 0  # instead of None (no 3150s sent when setpoint is False)
+        return heat_demand
+
     @property
     def window_open(self) -> Optional[bool]:  # 12B0
         return self._msg_value(_12B0, key=self.WINDOW_OPEN)
@@ -1661,7 +1783,7 @@ class HvacHumidity(BatteryState, Device):  # HUM (32) I/12(98|A0)
     The cardinal code is 12A0.
     """
 
-    _class = DEVICE_CLASS.HUM
+    _klass = DEV_KLASS.HUM
     _types = tuple()  # ("32",)
 
     REL_HUMIDITY = "relative_humidity"  # percentage (0.0-1.0)
@@ -1700,7 +1822,7 @@ class HvacSwitch(BatteryState, Device):  # SWI (39): I/22F[13]
     # RQ --- 32:166025 30:079129 --:------ 31DA 001 21
     # RP --- 30:079129 32:166025 --:------ 31DA 029 21EF00026036EF7FFF7FFF7FFF7FFF0002EF18FFFF000000EF7FFF7FFF
 
-    _class = DEVICE_CLASS.SWI
+    _klass = DEV_KLASS.SWI
     _types = tuple()  # ("39",)
 
     @property
@@ -1733,7 +1855,7 @@ class HvacVentilator(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
     # every /30
     # 30:079129 --:------ 30:079129 31D9 017 2100FF0000000000000000000000000000
 
-    _class = DEVICE_CLASS.FAN
+    _klass = DEV_KLASS.FAN
     _types = tuple()  # ("20", "37")
 
     @property
@@ -1774,43 +1896,35 @@ class HvacVentilator(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
         }
 
 
-_CLASS = "_class"
-DEVICE_BY_CLASS = {
-    getattr(c[1], _CLASS): c[1]
-    for c in getmembers(
-        modules[__name__],
-        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, _CLASS),
-    )
-}  # Every device class has a {klass: class}, e.g.: "CTL": Controller
-
-DEVICE_BY_ID_TYPE = {
-    k1: v2
-    for k1, v1 in _DEV_TYPE_TO_CLASS.items()
-    for k2, v2 in DEVICE_BY_CLASS.items()
-    if v1 == k2
-}  # e.g. "01": Controller,
-
-
-DEVICE_KLASS_BY_SIGNATURE = {
+OUT_DEV_KLASS_BY_SIGNATURE = {
     "FAN": ((RP, _31DA), (I_, _31D9), (I_, _31DA)),
     "SWI": ((I_, _22F1), (I_, _22F3)),
     "HUM": ((I_, _12A0)),
     "C02": ((I_, _1298)),
 }
 
+_DEV_CLASS_BY_KLASS = class_by_attr(__name__, "_klass")  # e.g. "CTL": Controller
 
-def create_device(gwy, dev_id, dev_class=None, **kwargs) -> Device:
+_DEV_CLASS_BY_ID_TYPE = {
+    k1: v2
+    for k1, v1 in _DEV_TYPE_TO_KLASS.items()
+    for k2, v2 in _DEV_CLASS_BY_KLASS.items()
+    if v1 == k2
+}  # e.g. "01": Controller,
+
+
+def create_device(gwy, dev_id: str, dev_klass=None, **kwargs) -> Device:
     """Create a device, and optionally perform discovery & start polling."""
 
     dev_addr = id_to_address(dev_id)
 
-    if dev_class is None:
+    if dev_klass is None:
         if dev_addr.type != "30":  # could be RFG or VNT  # DEX
-            dev_class = _DEV_TYPE_TO_CLASS.get(dev_addr.type, DEVICE_CLASS.GEN)  # DEX
+            dev_klass = _DEV_TYPE_TO_KLASS.get(dev_addr.type, DEV_KLASS.GEN)  # DEX
         else:
-            dev_class = DEVICE_CLASS.GEN  # generic
+            dev_klass = DEV_KLASS.GEN  # generic
 
-    device = DEVICE_BY_CLASS.get(dev_class, Device)(gwy, dev_addr, **kwargs)
+    device = _DEV_CLASS_BY_KLASS.get(dev_klass, Device)(gwy, dev_addr, **kwargs)
 
     if isinstance(gwy.pkt_protocol, PacketProtocolPort):
         device._start_discovery()
