@@ -4,9 +4,11 @@
 """RAMSES RF - a RAMSES-II protocol decoder & analyser."""
 
 import logging
+from inspect import getmembers, isclass
+from sys import modules
 from typing import List
 
-from .const import DHW_HACK, Discover, __dev_mode__
+from .const import Discover, __dev_mode__
 
 from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
@@ -87,6 +89,24 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
+def class_by_attr(name: str, attr: str) -> dict:
+    """Return a mapping of a (unique) attr of classes in a module to that class.
+
+    For example:
+      {"OTB": OtbGateway, "CTL": Controller}
+      {"RAD": RadZone, "UFH": UfhZone}
+      {"evohome": Evohome}
+    """
+
+    return {
+        getattr(c[1], attr): c[1]
+        for c in getmembers(
+            modules[name],
+            lambda m: isclass(m) and m.__module__ == name and hasattr(m, attr),
+        )
+    }
+
+
 def discover_decorator(fnc):
     # NOTE: only need to Wrap top-level entities
     def wrapper(self, discover_flag=Discover.ALL) -> None:
@@ -133,9 +153,6 @@ class Entity:
         pass
 
     def _handle_msg(self, msg) -> None:  # TODO: beware, this is a mess
-        if DHW_HACK and msg.code == _1260:
-            _LOGGER.debug(f"{msg._pkt} < handling (12)")  # HACK: lloyda
-
         if (
             self._gwy.pkt_protocol is None
             or msg.src.id != self._gwy.pkt_protocol._hgi80.get("device_id")
@@ -189,17 +206,14 @@ class Entity:
             _LOGGER.info(f"{cmd} < Sending is deprecated for {self}")
             return
 
-        if getattr(self, "has_battery", None):
-            _LOGGER.warning(f"{cmd} < Sending inadvisable for {self} (has a battery)")
+        if getattr(self, "has_battery", None) and cmd.dst.id == self.id:
+            _LOGGER.info(f"{cmd} < Sending inadvisable for {self} (has a battery)")
 
         cmd._source_entity = self
         # self._msgs.pop(cmd.code, None)  # NOTE: Cause of DHW bug
         self._gwy.send_cmd(cmd)
 
     def _msg_value(self, code, *args, **kwargs) -> dict:
-        if DHW_HACK and code == _1260:
-            _LOGGER.debug(f"{self._msgs.get(code)} < handling (20)")  # HACK: lloyda
-
         if isinstance(code, (str, tuple)):  # a code or a tuple of codes
             return self._msg_value_code(code, *args, **kwargs)
         return self._msg_value_msg(code, *args, **kwargs)  # assume is a Message
