@@ -10,9 +10,7 @@ import logging
 
 from .const import DONT_CREATE_ENTITIES, DONT_UPDATE_ENTITIES, __dev_mode__
 from .devices import Device
-from .protocol import Message
-from .protocol.exceptions import CorruptStateError
-from .protocol.ramses import RAMSES_CODES
+from .protocol import RAMSES_CODES, CorruptStateError, Message
 
 from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
@@ -83,10 +81,10 @@ CODE_NAMES = {k: v["name"] for k, v in RAMSES_CODES.items()}
 MSG_FORMAT_10 = "|| {:10s} | {:10s} | {:2s} | {:16s} | {:^4s} || {}"
 MSG_FORMAT_18 = "|| {:18s} | {:18s} | {:2s} | {:16s} | {:^4s} || {}"
 
-DEV_MODE = __dev_mode__ and True
+DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
-if False and DEV_MODE:
+if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
@@ -177,14 +175,9 @@ def process_msg(msg: Message) -> None:
             msg._gwy._prev_msg = msg
             return
 
-        if msg.src.type == "07":
-            _LOGGER.debug(f"{msg._pkt} < handling (01)")  # HACK: lloyda
-
         # _update_entities(msg, msg._gwy._prev_msg)  # update the state database
         if isinstance(msg.src, Device):
             msg.src._handle_msg(msg)
-        else:
-            _LOGGER.debug(f"{msg._pkt} < handling (02)")  # HACK: lloyda
 
         if msg.code not in (_0008, _0009, _3B00, _3EF1):  # special case: are fakeable
             msg._gwy._prev_msg = msg
