@@ -3,8 +3,13 @@
 #
 """RAMSES RF - a RAMSES-II protocol decoder & analyser."""
 
+# noqa: F401, pylint: disable=unused-import
+
+from logging import Logger
+
 from .address import is_valid_dev_id
-from .command import Command, FaultLog, Priority
+from .command import CODE_API_MAP, Command, FaultLog, Priority
+from .exceptions import CorruptStateError, ExpiredCallbackError
 from .logger import set_logger_timesource, set_pkt_logging
 from .message import Message
 from .packet import _PKT_LOGGER, Packet
@@ -71,6 +76,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _22F3,
     _2309,
     _2349,
+    _2389,
     _2D49,
     _2E04,
     _30C9,
@@ -88,3 +94,8 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _3EF1,
     _PUZZ,
 )
+
+
+def set_pkt_logging_config(**config) -> Logger:
+    set_pkt_logging(_PKT_LOGGER, **config)
+    return _PKT_LOGGER
