@@ -14,6 +14,7 @@ from typing import Any, Optional, Tuple
 
 from .address import Address
 from .exceptions import InvalidPacketError, InvalidPayloadError
+from .packet import fraction_expired
 from .parsers import PAYLOAD_PARSERS, parser_unknown
 from .ramses import CODE_IDX_COMPLEX, CODE_RQ_COMPLEX, RAMSES_CODES, RAMSES_DEVICES
 
@@ -63,6 +64,7 @@ from .const import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
     _2D49,
     _2E04,
     _30C9,
@@ -81,7 +83,7 @@ from .const import (  # noqa: F401, isort: skip
 
 __all__ = ["Message"]
 
-ATTR_ALIAS = "alias"  # duplicate
+HVAC_ONLY_CODES = (_1298, _12A0, _12C8, _22F1, _22F3, _31D9, _31DA, _31E0)
 
 CODE_NAMES = {k: v["name"] for k, v in RAMSES_CODES.items()}
 
@@ -98,9 +100,9 @@ if DEV_MODE:
 class Message:
     """The message class; will trap/log all invalid MSGs appropriately."""
 
-    CANT_EXPIRE = 0
-    HAS_EXPIRED = 1.2  # i.e. any value >= HAS_EXPIRED
+    CANT_EXPIRE = -1
     IS_EXPIRING = 0.8  # expected lifetime == 1.0
+    HAS_EXPIRED = 2.0  # incl. any value >= HAS_EXPIRED
 
     def __init__(self, gwy, pkt) -> None:
         """Create a message from a valid packet.
@@ -126,7 +128,7 @@ class Message:
         self._payload = self._validate(self._pkt.payload)  # ? raise InvalidPacketError
 
         self._str = None
-        self._expired_ = None
+        self._fraction_expired = None
         self._is_fragment = None
 
     def __repr__(self) -> str:
@@ -143,8 +145,8 @@ class Message:
             return ctx
 
         def display_name(addr: Address) -> str:  # TODO: is broken
-            # name = d.schema.get(ATTR_ALIAS) if self._gwy.config.use_aliases else None
-            name = None
+            # name = d.schema.get("alias") if self._gwy.config.use_aliases else None
+            name = None  # FIXME
             return name[:20] if name else Address._friendly(addr.id)
 
         if self._str is not None:
@@ -214,6 +216,7 @@ class Message:
             _0418: "log_idx",  # can be 2 DHW zones per system
             _10A0: "dhw_idx",  # can be 2 DHW zones per system
             _22C9: "ufh_idx",  # UFH circuit
+            _2389: "other_idx",  # anachronistic
             _2D49: "other_idx",  # non-evohome: hometronics
             _31D9: "hvac_id",
             _31DA: "hvac_id",
@@ -226,10 +229,13 @@ class Message:
         if self.code in (_3220,):  # FIXME: should be _SIMPLE
             return {}
 
+        #  I 068 03:201498 --:------ 03:201498 30C9 003 0106D6 # rare
+
         #  I --- 00:034798 --:------ 12:126457 2309 003 0201F4
         if not {self.src.type, self.dst.type} & {
             "01",
             "02",
+            "03",  # ?remove (see above, rare)
             "12",
             "18",
             "22",
@@ -242,6 +248,7 @@ class Message:
         if self.src.type == self.dst.type and self.src.type not in (
             "01",
             "02",
+            "03",  # ?remove (see above, rare)
             "18",
             "23",
         ):  # DEX
@@ -279,25 +286,26 @@ class Message:
     def _expired(self) -> Tuple[bool, Optional[bool]]:
         """Return True if the message is dated (does not require a valid payload)."""
 
-        if self._expired_ is not None:
-            if self._expired_ == self.CANT_EXPIRE:
+        if self._fraction_expired is not None:
+            if self._fraction_expired == self.CANT_EXPIRE:
                 return False
-            if self._expired_ >= self.HAS_EXPIRED * 2:  # TODO: should delete?
+            if self._fraction_expired > self.HAS_EXPIRED * 2:  # TODO: should delete?
                 return True
 
-        if self.code == _1F09 and self.verb == I_:
-            # RQs won't have remaining_seconds
-            # RP/Ws will have only partial cycle times
-            timeout = td(seconds=self.payload["remaining_seconds"])
-            self._expired_ = (self._gwy._dt_now() - self.dtm) / timeout
+        if self.code == _1F09 and self.verb != RQ:
+            # RQs won't have remaining_seconds, RP/Ws have only partial cycle times
+            self._fraction_expired = fraction_expired(
+                self._gwy._dt_now() - self.dtm,
+                td(seconds=self.payload["remaining_seconds"]),
+            )
         else:
-            self._expired_ = self._pkt._expired
+            self._fraction_expired = self._pkt._expired
 
-        if self._expired_ is False:  # treat as never expiring
-            _LOGGER.info("%s # cant expire", self._pkt)
-            self._expired_ = self.CANT_EXPIRE
+        if self._fraction_expired is False:  # treat as never expiring
+            _LOGGER.info(f"{self._pkt} # cant expire")
+            self._fraction_expired = self.CANT_EXPIRE
 
-        elif self._expired_ >= self.HAS_EXPIRED:  # TODO: should renew?
+        elif self._fraction_expired > self.HAS_EXPIRED:  # TODO: should renew?
             if any(
                 (
                     self.code == _1F09 and self.verb != I_,
@@ -306,14 +314,14 @@ class Message:
             ):
                 _logger = _LOGGER.info
             else:
-                _logger = _LOGGER.warning if DEV_MODE else _LOGGER.info
-            _logger("%s # has expired %s", self._pkt, f"({self._expired_ * 100:1.0f}%)")
+                _logger = _LOGGER.warning  # if DEV_MODE else _LOGGER.info  # TODO
+            _logger(f"{self._pkt} # has expired ({self._fraction_expired * 100:1.0f}%)")
 
-        # elif self._expired_ >= self.IS_EXPIRING:  # this could log multiple times
+        # elif self._fraction_expired >= self.IS_EXPIRING:  # this could log multiple times
         #     _LOGGER.error("%s # is expiring", self._pkt)
 
         # and self.dtm >= self._gwy._dt_now() - td(days=7)  # TODO: should be none >7d?
-        return self._expired_ >= self.HAS_EXPIRED
+        return self._fraction_expired > self.HAS_EXPIRED
 
     @property
     def _is_fragment_WIP(self) -> bool:
@@ -342,9 +350,10 @@ class Message:
         """
 
         try:  # parse the payload
-            _check_verb_code_payload(self, self._pkt.payload)  # ? InvalidPayloadError
-            _check_verb_code_src(self)  # ? InvalidPacketError
-            _check_verb_code_dst(self)  # ? InvalidPacketError
+            # TODO: only accept invalid packets to/from HGI when flag raised
+            _check_msg_payload(self, self._pkt.payload)  # ? InvalidPayloadError
+            _check_msg_src(self)  # ? InvalidPacketError
+            _check_msg_dst(self)  # ? InvalidPacketError
 
             if not self._has_payload or (
                 self.verb == RQ and self.code not in CODE_RQ_COMPLEX
@@ -365,20 +374,20 @@ class Message:
                 _LOGGER.exception
                 if DEV_MODE and self.src.type != "18"  # DEX
                 else _LOGGER.exception
-            )("%s << %s", self._pkt, f"{exc.__class__.__name__}({exc})")
+            )("%s < %s", self._pkt, f"{exc.__class__.__name__}({exc})")
 
         except InvalidPacketError as exc:
             (_LOGGER.exception if DEV_MODE else _LOGGER.warning)(
-                "%s << %s", self._pkt, exc
+                "%s < %s", self._pkt, exc
             )
 
         except (AttributeError, LookupError, TypeError, ValueError) as exc:  # TODO: dev
             _LOGGER.exception(
-                "%s << Coding error: %s", self._pkt, f"{exc.__class__.__name__}({exc})"
+                "%s < Coding error: %s", self._pkt, f"{exc.__class__.__name__}({exc})"
             )
 
         except NotImplementedError:  # parser_unknown (unknown packet code)
-            _LOGGER.warning("%s << Unknown packet code (cannot parse)", self._pkt)
+            _LOGGER.warning("%s < Unknown packet code (cannot parse)", self._pkt)
 
         else:
             # _LOGGER.error("%s", msg)
@@ -389,94 +398,126 @@ class Message:
 
 @lru_cache(maxsize=256)
 def re_compile_re_match(regex, string) -> bool:
+    # TODO: confirm this does speed things up
+    # Python has it's own caching of re.complile, _MAXCACHE = 512
+    # https://github.com/python/cpython/blob/3.10/Lib/re.py
     return re.compile(regex).match(string)
 
 
-def _check_verb_code_src(msg) -> None:
-    """Validate the packet's verb/code pair against its source device type.
+# cat *.log | grep -E ' (16:|20:|29:|32:|37:|39:|44:)'         | grep -vE ' (\*|RQ|042F|1060|10E0|22F1|3120|313F) ' | grep -vE ' (1298|12A0|12C8|22F3|31D9|31DA|31E0) '^C
+# cat *.log | grep -E ' (1298|12A0|12C8|22F3|31D9|31DA|31E0) ' | grep -vE ' (RQ|30:|16:|20:|29:|32:|37:|39:|44:)'
+
+
+def _check_msg_src(msg: Message) -> None:
+    """Validate the packet's source device type against its verb/code pair.
 
     Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
     """
-    if msg.src.type not in RAMSES_DEVICES:  # DEX  # TODO: fingerprint dev class
-        raise InvalidPacketError(f"Unknown src device type: {msg.src.id}")
 
-    if msg.src.type == "18":  # TODO: make a dynamic list if sensor/relay faking, DEX
-        if msg.code not in RAMSES_CODES:  # NOTE: HGI can send whatever it likes
-            raise InvalidPacketError(f"Unknown code for {msg.src.id} to Tx: {msg.code}")
+    #
+    #
+
+    if msg.src.type not in RAMSES_DEVICES:  # DEX, TODO: fingerprint dev class
+        if msg.code not in HVAC_ONLY_CODES:
+            raise InvalidPacketError(f"Unknown src device type: {msg.src.id}")
+        _LOGGER.warning(f"{msg._pkt} < Unknown src device type: {msg.src.id} (HVAC?)")
         return
 
-    # if msg.dst.type == "18":  # TODO: make a dynamic list if sensor/relay faking, DEX
-    #     return
+    #
+    #
+
+    #
+    #
 
     if msg.code not in RAMSES_DEVICES[msg.src.type]:  # DEX
-        raise InvalidPacketError(f"Invalid code for {msg.src.id} to Tx: {msg.code}")
+        if msg.src.type != "18":  # DEX
+            raise InvalidPacketError(f"Invalid code for {msg.src.id} to Tx: {msg.code}")
+        if msg.verb in (RQ, W_):  # DEX
+            return
+        _LOGGER.warning(f"{msg._pkt} < Invalid code for {msg.src.id} to Tx: {msg.code}")
+        return
 
+    #
+    #
+    #
+    #
+
+    #
+    # (code := RAMSES_DEVICES[msg.src.type][msg.code]) and msg.verb not in code:
     if msg.verb not in RAMSES_DEVICES[msg.src.type][msg.code]:  # DEX
         raise InvalidPacketError(
             f"Invalid verb/code for {msg.src.id} to Tx: {msg.verb}/{msg.code}"
         )
 
 
-def _check_verb_code_dst(msg) -> None:
-    """Validate the packet's verb/code pair against its destination device type.
+def _check_msg_dst(msg: Message) -> None:
+    """Validate the packet's destination device type against its verb/code pair.
 
     Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
     """
 
-    # check that the destination would normally respond...
-    if "18" in (msg.src.type, msg.dst.type) or msg.dst.type in ("--", "63"):  # DEX
-        return  # could omit this check to enforce strict checking
+    if msg.dst.type in ("--", "63"):  # or "18" in (msg.src.type, msg.dst.type):  # DEX
+        return  # TODO: HGI80s
 
-    if msg.dst.type not in RAMSES_DEVICES:  # DEX
-        raise InvalidPacketError(f"Unknown dst device type: {msg.dst.id}")
-
-    if msg.verb == I_:  # receiving an I isnt currently in the schema, so cant be tested
+    if msg.dst.type not in RAMSES_DEVICES:  # DEX, TODO: fingerprint dev class
+        if msg.code not in HVAC_ONLY_CODES:
+            raise InvalidPacketError(f"Unknown dst device type: {msg.dst.id}")
+        _LOGGER.warning(f"{msg._pkt} < Unknown dst device type: {msg.dst.id} (HVAC?)")
         return
 
-    # HACK: these exceptions-to-the-rule need sorting
+    if msg.verb == I_:  # TODO: not common, unless src=dst
+        return  # receiving an I isn't currently in the schema & cant yet be tested
+
     if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"01/{RQ}/{_3EF1}",):  # DEX
-        return
+        return  # HACK: an exception-to-the-rule that need sorting
 
     if msg.code not in RAMSES_DEVICES[msg.dst.type]:  # NOTE: is not OK for Rx, DEX
-        #  I --- 04:253797 --:------ 01:063844 1060 003 056401
-        # HACK: these exceptions-to-the-rule need sorting
-        # if msg.code in (_1060, ):
-        #     return
-        raise InvalidPacketError(f"Invalid code for {msg.dst.id} to Rx: {msg.code}")
+        if msg.dst.type != "18":  # NOTE: not (yet) needed because of 1st if, DEX
+            raise InvalidPacketError(f"Invalid code for {msg.dst.id} to Rx: {msg.code}")
+        if msg.verb == RP:  # DEX
+            return
+        _LOGGER.warning(f"{msg._pkt} < Invalid code for {msg.dst.id} to Tx: {msg.code}")
+        return
 
-    # HACK: these exceptions-to-the-rule need sorting
     if f"{msg.verb}/{msg.code}" in (f"{W_}/{_0001}",):
-        return
-
-    # HACK: these exceptions-to-the-rule need sorting
+        return  # HACK: an exception-to-the-rule that need sorting
     if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"13/{RQ}/{_3EF0}",):  # DEX
-        return
+        return  # HACK: an exception-to-the-rule that need sorting
 
     verb = {RQ: RP, RP: RQ, W_: I_}[msg.verb]
+    # (code := RAMSES_DEVICES[msg.dst.type][msg.code]) and verb not in code:
     if verb not in RAMSES_DEVICES[msg.dst.type][msg.code]:  # DEX
         raise InvalidPacketError(
             f"Invalid verb/code for {msg.dst.id} to Rx: {msg.verb}/{msg.code}"
         )
 
 
-def _check_verb_code_payload(msg, payload) -> None:
-    """Validate the packet's verb/code pair against its payload.
+def _check_msg_payload(msg: Message, payload) -> None:
+    """Validate the packet's payload against its verb/code pair.
 
     Raise an InvalidPayloadError if the payload is invalid, otherwise simply return.
-    Some parsers may also raise InvalidPayloadError (e.g. 3220).
+
+    The HGI80-compatible devices can do what they like, but a warning is logged.
+    Some parsers may also raise InvalidPayloadError (e.g. 3220), albeit later on.
     """
 
-    # if self.code not in RAMSES_CODES:
-    #     raise InvalidPacketError(f"Unknown code: {msg.code}")
-
     try:
-        regex = RAMSES_CODES[msg.code][msg.verb]
-        if not re_compile_re_match(regex, payload) and msg.src.type != "18":  # DEX
-            raise InvalidPayloadError(f"Payload doesn't match '{regex}': {payload}")
-    except KeyError:
-        if "18" not in (msg.src.type, msg.dst.type):  # DEX
+        if msg.code not in RAMSES_CODES:
+            raise InvalidPacketError(f"Unknown code: {msg.code}")
+
+        try:
+            regex = RAMSES_CODES[msg.code][msg.verb]
+        except KeyError:
             raise InvalidPacketError(f"Unknown verb/code pair: {msg.verb}/{msg.code}")
 
+        if not re_compile_re_match(regex, payload):
+            raise InvalidPayloadError(f"Payload doesn't match '{regex}': {payload}")
+
+    except InvalidPacketError as exc:  # incl. InvalidPayloadError
+        if "18" not in (msg.src.type, msg.dst.type):  # DEX, HGI80 can do what it likes
+            raise exc  # TODO: messy - these msgs not ignore
+            _LOGGER.warning(f"{msg._pkt} < {exc}")
+
     # TODO: put this back, or leave it to the parser?
     # if msg.code == _3220:
     #     msg_id = int(payload[4:6], 16)
