@@ -116,6 +116,7 @@ LOOKUP_PUZZ = {
     "11": "impersonating",  # pkt header, e.g. 30C9| I|03:123001 (15 characters, packed)
     "12": "message",  # .   # message only, max len is 16 ascii characters
     "13": "message",  # .   # message only, but without a timestamp, max len 22 chars
+    "7F": "null",  # .      # packet is null / was nullified: payload to be ignored
 }  # "00" is reserved
 
 DEV_MODE = __dev_mode__ and False
@@ -540,8 +541,9 @@ def parser_0418(payload, msg) -> Optional[dict]:
     if payload[10:12] == "FC" and result["device_class"] == "actuator":
         result["device_class"] = ATTR_HTG_CONTROL  # aka Boiler relay
 
+    # 1C: 'Comms fault, Actuator': seen with boiler relays
     assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
-        payload[10:12] in ("F9", "FA", "FC")  # "1C"?
+        payload[10:12] in ("1C", "F9", "FA", "FC")
     ), f"unexpected domain_id: {payload[10:12]}"
 
     if payload[12:14] != "00":  # TODO: Controller
@@ -561,7 +563,7 @@ def parser_0418(payload, msg) -> Optional[dict]:
         {
             "_unknown_1": payload[6:8],  # B0 ?priority
             "_unknown_2": payload[14:18],  # 0000
-            "_unknown_3": payload[30:38],  # FFFF7000
+            "_unknown_3": payload[30:38],  # FFFF7000/1/2
         }
     )
 
@@ -810,6 +812,7 @@ def parser_10e0(payload, msg) -> Optional[dict]:
                 "0001001B361B01FEFF",  # CVE-RF
                 "0001001B381B01FEFF",  # CVE-RF
                 "00010028080101FEFF",  # VMS-12C39
+                "0001C822060166FEFF",  # VMS-17C01
             ), payload[2:20]
         else:
             assert False, payload[2:20]
@@ -1722,6 +1725,20 @@ def parser_3220(payload, msg) -> Optional[dict]:
 
             result.update(ot_value)
 
+        try:
+            assert ot_id != 0 or (
+                [result["value"][i] for i in (2, 3, 4, 5, 6, 7)] == [0] * 6
+            ), result["value"]
+
+            assert ot_id != 0 or (
+                [result["value"][8 + i] for i in (0, 4, 5, 6, 7)] == [0] * 5
+            ), result["value"]
+        except AssertionError:
+            _LOGGER.warning(
+                f"{msg._pkt} < Support development by reporting this pkt, "
+                "please include a description of your system"
+            )
+
     result[MSG_DESC] = ot_schema.get(EN)
     return result
 
@@ -1804,7 +1821,7 @@ def parser_3ef0(payload, msg) -> dict:
 
     result = {
         "modulation_level": mod_level,
-        "_flags_0": payload[4:6],
+        "_flags_2": payload[4:6],
     }
 
     if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
@@ -1818,9 +1835,9 @@ def parser_3ef0(payload, msg) -> dict:
         result.update(
             {
                 "_flags_3": flag8(payload[6:8]),
-                "ch_enabled": bool(int(payload[6:8], 0x10) & 1 << 1),
+                "ch_active": bool(int(payload[6:8], 0x10) & 1 << 1),
                 "dhw_active": bool(int(payload[6:8], 0x10) & 1 << 2),
-                "flame_active": bool(int(payload[6:8], 0x10) & 1 << 3),
+                "flame_active": bool(int(payload[6:8], 0x10) & 1 << 3),  # flame_on
                 "_unknown_4": payload[8:10],
                 "_unknown_5": payload[10:12],  # rel_modulation?
             }
@@ -1835,12 +1852,25 @@ def parser_3ef0(payload, msg) -> dict:
         result.update(
             {
                 "_flags_6": flag8(payload[12:14]),
-                "ch_active": bool(int(payload[12:14], 0x10) & 1 << 0),
+                "ch_enabled": bool(int(payload[12:14], 0x10) & 1 << 0),
                 "ch_setpoint": int(payload[14:16], 0x10),
                 "max_rel_modulation": percent(payload[16:18], high_res=False),
             }
         )
 
+    try:
+        assert "_flags_3" not in result or (
+            [result["_flags_3"][i] for i in (0, 1, 2, 3)] == [0] * 4
+        ), result["_flags_3"]
+        assert "_flags_6" not in result or (
+            [result["_flags_6"][i] for i in (0, 1, 2, 3, 4, 5)] == [0] * 6
+        ), result["_flags_6"]
+    except AssertionError:
+        _LOGGER.warning(
+            f"{msg._pkt} < Support development by reporting this pkt, "
+            "please include a description of your system"
+        )
+
     return result
 
 
@@ -1911,7 +1941,7 @@ def parser_7fff(payload, msg) -> Optional[dict]:
         dtm = dt.fromtimestamp(int(payload[4:16], 16) / 1000)
         result["datetime"] = dtm.isoformat(timespec="milliseconds")
 
-    msg_type = LOOKUP_PUZZ.get(payload[2:4], "message")
+    msg_type = LOOKUP_PUZZ.get(payload[2:4], "unknown")
 
     if payload[2:4] == "11":
         msg = str_from_hex(payload[16:])
@@ -1920,6 +1950,9 @@ def parser_7fff(payload, msg) -> Optional[dict]:
     elif payload[2:4] == "13":
         result[msg_type] = str_from_hex(payload[4:])
 
+    elif payload[2:4] == "7F":
+        result[msg_type] = payload[4:]
+
     else:
         result[msg_type] = str_from_hex(payload[16:])
 
