@@ -6,8 +6,9 @@
 import logging
 from datetime import timedelta as td
 
-from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .const import DEV_KLASS, DEVICE_TYPE_BY_CLASS
 
+from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
 
 from .const import (  # noqa: F401, isort: skip
     _0001,
@@ -22,6 +23,7 @@ from .const import (  # noqa: F401, isort: skip
     _000E,
     _0016,
     _0100,
+    _0150,
     _01D0,
     _01E9,
     _0404,
@@ -30,9 +32,13 @@ from .const import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
+    _1098,
     _10A0,
+    _10B0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -42,9 +48,12 @@ from .const import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
+    _1FD0,
     _1FD4,
     _2249,
     _22C9,
@@ -54,6 +63,11 @@ from .const import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
+    _2400,
+    _2401,
+    _2410,
+    _2420,
     _2D49,
     _2E04,
     _30C9,
@@ -63,7 +77,11 @@ from .const import (  # noqa: F401, isort: skip
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
+    _3221,
+    _3223,
     _3B00,
     _3EF0,
     _3EF1,
@@ -94,11 +112,10 @@ RAMSES_CODES = {  # rf_unknown
         I_: r"^00FFFF02(00|FF)$",  # loopback
         W_: r"^(0[0-9A-F]|F[CF])000005(01|05)$",
     },  # TODO: there appears to be a dodgy? RQ/RP for UFC
-    _0002: {  # WIP: outdoor_sensor
+    _0002: {  # WIP: outdoor_sensor - CODE_IDX_COMPLEX?
         NAME: "outdoor_sensor",
         I_: r"^0[0-4][0-9A-F]{4}(00|01|02|05)$",  # Domoticz sends ^02!!
         RQ: r"^00$",  # NOTE: sent by an RFG100
-        RP: r"^00[0-9A-F]{4}(00|01)$",  # 007FFF00 is null resp?
     },
     _0004: {  # zone_name
         NAME: "zone_name",
@@ -111,7 +128,7 @@ RAMSES_CODES = {  # rf_unknown
         #  I --- 34:092243 --:------ 34:092243 0005 012 000A0000-000F0000-00100000
         I_: r"^(00[01][0-9A-F]{5}){1,3}$",
         RQ: r"^00[01][0-9A-F]$",  # f"00{zone_type}", evohome wont respond to 00
-        RP: r"^00[01][0-9A-F]{5}$",
+        RP: r"^00[01][0-9A-F]{3,5}$",
         EXPIRES: False,
     },
     _0006: {  # schedule_sync  # TODO: what for DHW schedule?
@@ -137,8 +154,9 @@ RAMSES_CODES = {  # rf_unknown
     _000A: {  # zone_params
         NAME: "zone_params",
         I_: r"^(0[0-9A-F][0-9A-F]{10}){1,8}$",
+        W_: r"^0[0-9A-F][0-9A-F]{10}$",
         RQ: r"^0[0-9A-F]((00)?|([0-9A-F]{10})+)$",  # is: r"^0[0-9A-F]([0-9A-F]{10})+$"
-        RP: r"^0[0-9A-F]([0-9A-F]{10})+$",  # null_rp: xx,007FFF7FFF
+        RP: r"^0[0-9A-F][0-9A-F]{10}$",  # null_rp: xx/007FFF7FFF
         # 17:54:13.126 063 RQ --- 34:064023 01:145038 --:------ 000A 001 03
         # 17:54:13.141 045 RP --- 01:145038 34:064023 --:------ 000A 006 031002260B86
         # 19:20:49.460 062 RQ --- 12:010740 01:145038 --:------ 000A 006 080001F40DAC
@@ -167,6 +185,11 @@ RAMSES_CODES = {  # rf_unknown
         RP: r"^00[0-9A-F]{4}F{4}$",
         EXPIRES: td(days=1),  # TODO: make longer?
     },
+    _0150: {  # unknown
+        NAME: "message_0150",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
     _01D0: {  # TODO: not well understood, definitely a real code, zone_idx is a guess
         NAME: "message_01d0",
         I_: r"^0[0-9A-F][0-9A-F]{2}$",
@@ -183,18 +206,24 @@ RAMSES_CODES = {  # rf_unknown
     },
     _0404: {  # zone_schedule
         NAME: "zone_schedule",
-        RP: r"^0[0-9A-F](20|23)0008[0-9A-F]{6}[0-9A-F]{2,82}$",
+        I_: r"^0[0-9A-F](20|23)0008[0-9A-F]{6}$",
         RQ: r"^0[0-9A-F](20|23)000800[0-9A-F]{4}$",
+        RP: r"^0[0-9A-F](20|23)0008[0-9A-F]{6}[0-9A-F]{2,82}$",
+        W_: r"^0[0-9A-F](20|23)0008[0-9A-F]{6}[0-9A-F]{2,82}$",  # as per RP
+        EXPIRES: None,
     },
     _0418: {  # system_fault
         NAME: "system_fault",
-        I_: r"^00(00|40|C0)[0-3][0-9A-F]B0[0-9A-F]{6}0000[0-9A-F]{12}FFFF7000[0-9A-F]{6}$",
+        I_: r"^00(00|40|C0)[0-3][0-9A-F]B0[0-9A-F]{6}0000[0-9A-F]{12}FFFF700[012][0-9A-F]{6}$",
         RQ: r"^0000[0-3][0-9A-F]$",  # f"0000{log_idx}", no payload
     },
     _042F: {  # unknown, # non-evohome are len==9, seen only once?
         # .I --- 32:168090 --:------ 32:168090 042F 009 000000100F00105050
+        # RP --- 10:048122 18:006402 --:------ 042F 009 000200001400163010
         NAME: "message_042f",
-        I_: r"^00([0-9A-F]{2}){7}$",
+        I_: r"^00([0-9A-F]{2}){7,8}$",
+        RQ: r"^00$",
+        RP: r"^00([0-9A-F]{2}){7,8}$",
     },
     _0B04: {  # unknown
         #  I --- --:------ --:------ 12:207082 0B04 002 00C8
@@ -208,39 +237,61 @@ RAMSES_CODES = {  # rf_unknown
     },
     _1060: {  # device_battery
         NAME: "device_battery",
-        I_: r"^0[0-9A-F](FF|[0-9A-F]{2})0[01]$",
+        I_: r"^0[0-9A-F](FF|[0-9A-F]{2})0[01]$",  # HCW: r"^(FF|0[0-9A-F]...
         EXPIRES: td(days=1),
     },
+    _1081: {  # max_ch_setpoint
+        NAME: "max_ch_setpoint",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{4}$",
+    },
     _1090: {  # unknown
         NAME: "message_1090",
         # RQ: r"^00$",  # TODO:
     },
+    _1098: {  # unknown
+        NAME: "message_1098",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _10B0: {  # unknown
+        NAME: "message_10b0",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
     _10A0: {  # dhw_params
         NAME: "dhw_params",
         # RQ --- 07:045960 01:145038 --:------ 10A0 006 0013740003E4
+        # RP --- 10:048122 18:006402 --:------ 10A0 003 001B58
         # NOTE: RFG100 uses a domain id! (00|01)
         # 19:14:24.662 051 RQ --- 30:185469 01:037519 --:------ 10A0 001 00
         # 19:14:31.463 053 RQ --- 30:185469 01:037519 --:------ 10A0 001 01
-        I_: r"^0[01]([0-9A-F]{10})?$",  # NOTE: RQ/07/10A0 has a payload
+        I_: r"^0[01][0-9A-F]{4}([0-9A-F]{6})?$",  # NOTE: RQ/07/10A0 has a payload
         RQ: r"^0[01]([0-9A-F]{10})?$",  # NOTE: RQ/07/10A0 has a payload
         EXPIRES: td(hours=4),
     },
     _10E0: {  # device_info
         NAME: "device_info",
-        I_: r"^00[0-9A-F]{30,}$",  # NOTE: RP is same
+        I_: r"^00[0-9A-F]{30,}$",  # r"^[0-9A-F]{32,}$" might be OK
         RQ: r"^00$",  # NOTE: will accept [0-9A-F]{2}
         # RP: r"^[0-9A-F]{2}([0-9A-F]){30,}$",  # NOTE: indx same as RQ
         EXPIRES: False,
     },
+    _10E1: {  # device_id
+        NAME: "device_id",
+        RP: r"^00[0-9A-F]{6}$",
+        RQ: r"^00$",
+        EXPIRES: False,
+    },
     _1100: {  # tpi_params
         NAME: "tpi_params",
         # RQ --- 01:145038 13:163733 --:------ 1100 008 00180400007FFF01  # boiler relay
         # RP --- 13:163733 01:145038 --:------ 1100 008 00180400FF7FFF01
         # RQ --- 01:145038 13:035462 --:------ 1100 008 FC240428007FFF01  # not bolier relay
         # RP --- 13:035462 01:145038 --:------ 1100 008 00240428007FFF01
-        I_: r"^(00|FC)[0-9A-F]{6}(00|FF)([0-9A-F]{4}01)+$",
-        RQ: r"^(00|FC)([0-9A-F]{6}(00|FF)([0-9A-F]{4}01)+)?$",  # RQ/13:/00, or RQ/01:/FC:
-        W_: r"^(00|FC)[0-9A-F]{6}(00|FF)([0-9A-F]{4}01)+$",  # TODO: is there no I?
+        I_: r"^(00|FC)[0-9A-F]{6}(00|FF)([0-9A-F]{4}01)?$",
+        W_: r"^(00|FC)[0-9A-F]{6}(00|FF)([0-9A-F]{4}01)?$",  # TODO: is there no I?
+        RQ: r"^(00|FC)([0-9A-F]{6}(00|FF)([0-9A-F]{4}01)?)?$",  # RQ/13:/00, or RQ/01:/FC:
         EXPIRES: td(days=1),
     },
     _1260: {  # dhw_temp
@@ -256,6 +307,7 @@ RAMSES_CODES = {  # rf_unknown
     },
     _1280: {  # outdoor_humidity
         NAME: "outdoor_humidity",
+        I_: r"^00[0-9A-F]{10}$",
     },
     _1290: {  # outdoor_temp
         NAME: "outdoor_temp",
@@ -279,12 +331,22 @@ RAMSES_CODES = {  # rf_unknown
     },
     _12C0: {  # displayed_temp (HVAC room temp)
         NAME: "displayed_temp",  # displayed room temp
-        I_: r"^00[0-9A-F]{2}01$",
+        I_: r"^00[0-9A-F]{2}0[01]$",
     },
     _12C8: {  # hvac_12C8 - %?
         NAME: "hvac_12C8",
         I_: r"^0000[0-9A-F]{2}$",
     },
+    _12F0: {  # unknown
+        NAME: "message_12f0",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _1300: {  # cv water pressure (usu. for ch)
+        NAME: "ch_pressure",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{4}$",
+    },
     _1F09: {  # system_sync - "FF" (I), "00" (RP), "F8" (W, after 1FC9)
         NAME: "system_sync",
         I_: r"^(00|01|DB|FF)[0-9A-F]{4}$",  # FF is evohome, DB is Hometronics
@@ -309,6 +371,11 @@ RAMSES_CODES = {  # rf_unknown
         I_: r"^((0[0-9A-F]|F[9ABCF])([0-9A-F]{10}))+$",
         W_: r"^((0[0-9A-F]|F[9ABCF])([0-9A-F]{10}))+$",
     },
+    _1FD0: {  # unknown
+        NAME: "message_1fd0",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
     _1FD4: {  # opentherm_sync
         NAME: "opentherm_sync",
         I_: r"^00([0-9A-F]{4})$",
@@ -322,7 +389,7 @@ RAMSES_CODES = {  # rf_unknown
         #  I --- 02:001107 --:------ 02:001107 22C9 006 04-0834-0A28-01
         NAME: "ufh_setpoint",
         I_: r"^(0[0-9A-F][0-9A-F]{8}01){1,4}$",  # ~000A array, but max_len 24, not 48!
-        # RQ: None?,
+        # RP: Appear wont get any?,
     },
     _22D0: {  # HVAC system switch
         NAME: "message_22d0",
@@ -339,7 +406,7 @@ RAMSES_CODES = {  # rf_unknown
     },
     _22F3: {  # switch_duration
         NAME: "switch_duration",
-        I_: r"^0000[0-9A-F]{2}$",
+        I_: r"^0000[0-9A-F]{2}([0-9A-F]{8})?$",
     },  # minutes
     _2309: {  # setpoint
         NAME: "setpoint",
@@ -359,6 +426,31 @@ RAMSES_CODES = {  # rf_unknown
         # RQ --- 22:070483 01:063844 --:------ 2349 007 06-0708-03-000027
         EXPIRES: td(hours=4),
     },
+    _2389: {  # unknown - CODE_IDX_COMPLEX?
+        #  I 024 03:052382 --:------ 03:052382 2389 003 02001B
+        NAME: "unknown_2389",
+        I_: r"^0[0-4][0-9A-F]{4}$",
+    },
+    _2400: {  # unknown
+        NAME: "message_2400",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _2401: {  # unknown
+        NAME: "message_2401",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _2410: {  # unknown
+        NAME: "message_2410",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _2420: {  # unknown
+        NAME: "message_2420",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
     _2D49: {  # unknown
         NAME: "message_2d49",
         # 10:14:08.526 045  I --- 01:023389 --:------ 01:023389 2D49 003 010000
@@ -414,13 +506,33 @@ RAMSES_CODES = {  # rf_unknown
     },
     _31E0: {  # ext_ventilation - External Ventilation Status
         NAME: "ext_ventilation",
-        I_: r"^0000(00|C8)00$",
+        I_: r"^0000(00|C8)(00|FF)$",
+    },
+    _3200: {  # boiler output temp
+        NAME: "boiler_output",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{4}$",
+    },
+    _3210: {  # boiler return temp
+        NAME: "boiler_return",
+        RQ: r"^00$",
+        RP: r"^00[0-9A-F]{4}$",
     },
     _3220: {  # opentherm_msg
         NAME: "opentherm_msg",
         RQ: r"^00[0-9A-F]{4}0{4}$",  # is strictly: r"^00[0-9A-F]{8}$",
         RP: r"^00[0-9A-F]{8}$",
     },
+    _3221: {  # unknown
+        NAME: "message_3221",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
+    _3223: {  # unknown
+        NAME: "message_3223",
+        RQ: r"^00$",
+        RP: r"^00",
+    },
     _3B00: {  # actuator_sync, NOTE: no RQ
         NAME: "actuator_sync",
         I_: r"^(00|FC)(00|C8)$",
@@ -447,7 +559,7 @@ RAMSES_CODES = {  # rf_unknown
     },
     _PUZZ: {
         NAME: "puzzle_packet",
-        I_: r"^00([0-9A-F]+7F)+",
+        I_: r"^00(([0-9A-F]){2})+$",
     },
 }
 for code in RAMSES_CODES.values():
@@ -457,7 +569,9 @@ for code in RAMSES_CODES.values():
 CODE_ONLY_FROM_CTL = [_1030, _1F09, _22D0, _313F]  # I packets, TODO: 31Dx too?
 
 #
+# I --- 01:210309 --:------ 01:210309 0009 006 FC00FFF900FF
 CODES_WITH_ARRAYS = {
+    _0005: [4, ("34",)],
     _0009: [3, ("01", "12", "22")],
     _000A: [6, ("01", "12", "22")],
     _2309: [3, ("01", "12", "22")],
@@ -465,7 +579,7 @@ CODES_WITH_ARRAYS = {
     _2249: [7, ("23",)],
     _22C9: [6, ("02",)],
     _3150: [2, ("02",)],
-}  # element_length, src.type(s) (and dst.type too)
+}  # TODO dex: element_length, src.type(s) (and dst.type too)
 
 #
 CODE_RQ_COMPLEX = [
@@ -479,7 +593,7 @@ CODE_RQ_COMPLEX = [
     _1100,  # optional payload
     _2309,  # optional payload
     _2349,  # optional payload
-    _3120,  # context: msg_id, and payload
+    _3220,  # context: msg_id, and payload
 ]
 # CODE_RQ_COMPLEX = []
 RQ_NO_PAYLOAD = [
@@ -509,7 +623,7 @@ CODE_RQ_UNKNOWN.sort()  # or print(f"unknown  = {list(CODE_RQ_UNKNOWN)}\r\n")
 
 # IDX_COMPLEX - *usually has* a context, but doesn't satisfy criteria for IDX_SIMPLE:
 # all known codes are in one of IDX_COMPLEX, IDX_NONE, IDX_SIMPLE
-CODE_IDX_COMPLEX = [_0005, _000C, _1100, _3220]  # TODO: move 0005 to ..._NONE?
+CODE_IDX_COMPLEX = [_0005, _000C, _1100, _3220]  # TODO: 0005 to ..._NONE?
 
 # IDX_SIMPLE - *can have* a context, but sometimes not (usu. 00): only ever payload[:2],
 # either a zone_idx, domain_id or (UFC) circuit_idx (or array of such, i.e. seqx[:2])
@@ -532,7 +646,7 @@ CODE_IDX_NONE = [
     if k not in CODE_IDX_COMPLEX + CODE_IDX_SIMPLE
     and ((RQ in v and v[RQ][:3] == "^00") or (I_ in v and v[I_][:3] == "^00"))
 ]
-CODE_IDX_NONE.extend((_0001, _2E04, _31DA, _PUZZ))  # 31DA does appear to have an idx?
+CODE_IDX_NONE.extend((_0002, _2389, _2E04, _31DA))  # 31DA does appear to have an idx?
 #
 #
 _CODE_IDX_UNKNOWN = [
@@ -551,17 +665,17 @@ CODE_IDX_DOMAIN = {  # not necc. mutex
     _3B00: "^FC",
 }
 #
-CODE_IDX_COMPLEX.sort()  # or print(f"complex = {CODE_IDX_COMPLEX}")  # TODO: remove
-CODE_IDX_NONE.sort()  # or print(f"none    = {CODE_IDX_NONE}")  # TODO: remove
-CODE_IDX_SIMPLE.sort()  # or print(f"simple  = {CODE_IDX_SIMPLE}")  # TODO: remove
-_CODE_IDX_UNKNOWN.sort()  # or print(f"unknown = {_CODE_IDX_UNKNOWN}")  # TODO: remove
+CODE_IDX_COMPLEX.sort()  # or print(f"complex = {CODE_IDX_COMPLEX}")
+CODE_IDX_NONE.sort()  # or print(f"none    = {CODE_IDX_NONE}")
+CODE_IDX_SIMPLE.sort()  # or print(f"simple  = {CODE_IDX_SIMPLE}")
+_CODE_IDX_UNKNOWN.sort()  # or print(f"unknown = {_CODE_IDX_UNKNOWN}")
 # print(f"domains = {list(CODE_IDX_DOMAIN)}")
 
 ################
 # RAMSES_DEVICES
 #
-RAMSES_DEVICES = {
-    "01": {  # e.g. ATC928: Evohome Colour Controller
+RAMSES_DEVICES_CLASS = {
+    DEV_KLASS.CTL: {  # e.g. ATC928: Evohome Colour Controller
         _0001: {W_: {}},
         _0002: {I_: {}, RP: {}},
         _0004: {I_: {}, RP: {}},
@@ -600,7 +714,7 @@ RAMSES_DEVICES = {
         _3B00: {I_: {}},
         _3EF0: {RQ: {}},
     },
-    "02": {  # e.g. HCE80/HCC80: Underfloor Heating Controller
+    DEV_KLASS.UFC: {  # e.g. HCE80/HCC80: Underfloor Heating Controller
         _0001: {RP: {}, W_: {}},  # TODO: Ix RP
         _0005: {RP: {}},
         _0008: {I_: {}},
@@ -613,18 +727,7 @@ RAMSES_DEVICES = {
         _2309: {RP: {}},
         _3150: {I_: {}},
     },
-    "03": {  # e.g. HCF82/HCW82: Room Temperature Sensor
-        _0001: {W_: {}},
-        _0008: {I_: {}},
-        _0009: {I_: {}},
-        _1060: {I_: {}},
-        _1100: {I_: {}},
-        _1F09: {I_: {}},
-        _1FC9: {I_: {}},
-        _2309: {I_: {}},
-        _30C9: {I_: {}},
-    },
-    "04": {  # e.g. HR92/HR91: Radiator Controller
+    DEV_KLASS.TRV: {  # e.g. HR92/HR91: Radiator Controller
         _0001: {W_: {r"^0[0-9A-F]"}},
         _0004: {RQ: {r"^0[0-9A-F]00$"}},
         _0016: {RQ: {}, RP: {}},
@@ -641,35 +744,87 @@ RAMSES_DEVICES = {
         _313F: {RQ: {r"^00$"}},
         _3150: {I_: {r"^0[0-9A-F]{3}$"}},
     },
-    "07": {  # e.g. CS92: (DHW) Cylinder Thermostat
+    DEV_KLASS.DHW: {  # e.g. CS92: (DHW) Cylinder Thermostat
         _0016: {RQ: {}},
         _1060: {I_: {}},
         _10A0: {RQ: {}},  # This RQ/07/10A0 includes a payload
         _1260: {I_: {}},
         _1FC9: {I_: {}},
     },
-    "08": {
+    DEV_KLASS.OTB: {  # e.g. R8810/R8820: OpenTherm Bridge
+        _0009: {I_: {}},  # 1/24h for a R8820 (not an R8810)
+        _0150: {RP: {}},  # R8820A only?
+        _042F: {I_: {}, RP: {}},
+        _1081: {RP: {}},  # R8820A only?
+        _1098: {RP: {}},  # R8820A only?
+        _10A0: {RP: {}},
+        _10B0: {RP: {}},  # R8820A only?
+        _10E0: {I_: {}, RP: {}},
+        _10E1: {RP: {}},  # R8820A only?
+        _1260: {RP: {}},
+        _1290: {RP: {}},
+        _12F0: {RP: {}},  # R8820A only?
+        _1300: {RP: {}},  # R8820A only?
+        _1FC9: {I_: {}, W_: {}},
+        _1FD0: {RP: {}},  # R8820A only?
+        _1FD4: {I_: {}},  # 2/min fro R8810, every ~210 sec for R8820
+        _22D9: {RP: {}},
+        _2400: {RP: {}},  # R8820A only?
+        _2401: {RP: {}},  # R8820A only?
+        _2410: {RP: {}},  # R8820A only?
+        _2420: {RP: {}},  # R8820A only?
+        _3150: {I_: {}},
+        _3200: {RP: {}},  # R8820A only?
+        _3210: {RP: {}},  # R8820A only?
+        _3220: {RP: {}},
+        _3221: {RP: {}},  # R8820A only?
+        _3223: {RP: {}},  # R8820A only?
+        _3EF0: {I_: {}, RP: {}},
+        _3EF1: {RP: {}},
+    },  # see: https://www.opentherm.eu/request-details/?post_ids=2944
+    DEV_KLASS.BDR: {  # e.g. BDR91A/BDR91T: Wireless Relay Box
+        _0008: {RP: {}},  # doesn't RP/0009
+        _0016: {RP: {}},
+        # _10E0: {},  # 13: will not RP/10E0 # TODO: how to indicate that fact here
+        _1100: {I_: {}, RP: {}},
+        _1FC9: {RP: {}, W_: {}},
+        _3B00: {I_: {}},
+        _3EF0: {I_: {}},
+        # RP: {},  # RQ --- 01:145038 13:237335 --:------ 3EF0 001 00
+        _3EF1: {RP: {}},
+    },
+    DEV_KLASS.EXT: {
+        _0002: {I_: {}},
+        _1FC9: {I_: {}},
+    },  # i.e. HB85 (ext. temperature/luminosity(lux)), HB95 (+ wind speed)
+    DEV_KLASS.JIM: {  # Jasper Interface Module, 08
         _0008: {RQ: {}},
         _10E0: {I_: {}},
         _1100: {I_: {}},
         _3EF0: {I_: {}},
         _3EF1: {RP: {}},
     },
-    "10": {  # e.g. R8810/R8820: OpenTherm Bridge
-        _0009: {I_: {}},  # 1/24h for a R8820 (not an R8810)
-        _10A0: {RP: {}},
-        _10E0: {I_: {}, RP: {}},
-        _1260: {RP: {}},
-        _1290: {RP: {}},
-        _1FC9: {I_: {}, W_: {}},
-        _1FD4: {I_: {}},
-        _22D9: {RP: {}},
-        _3150: {I_: {}},
-        _3220: {RP: {}},
-        _3EF0: {I_: {}, RP: {}},
-        _3EF1: {RP: {}},
-    },  # see: https://www.opentherm.eu/request-details/?post_ids=2944
-    "12": {  # e.g. DTS92(E): Digital Room Thermostat
+    DEV_KLASS.JST: {  # Jasper Stat, 31
+        _0008: {I_: {}},
+        _10E0: {I_: {}},
+        _3EF1: {RQ: {}, RP: {}},
+    },
+}
+RAMSES_DEVICES = {
+    "03": {  # e.g. HCF82/HCW82: Room Temperature Sensor
+        _0001: {W_: {}},
+        _0002: {I_: {}},
+        _0008: {I_: {}},
+        _0009: {I_: {}},
+        _1060: {I_: {}},
+        _1100: {I_: {}},
+        _1F09: {I_: {}},
+        _1FC9: {I_: {}},
+        _2309: {I_: {}},
+        _2389: {I_: {}},
+        _30C9: {I_: {}},
+    },
+    "12": {  # DTS92(E): Digital Room Thermostat
         _0001: {W_: {}},
         _0008: {I_: {}},
         _0009: {I_: {}},
@@ -689,33 +844,9 @@ RAMSES_DEVICES = {
         _3B00: {I_: {}},
         _3EF1: {RQ: {}},
     },
-    "13": {  # e.g. BDR91A/BDR91T: Wireless Relay Box
-        _0008: {RP: {}},  # doesn't RP/0009
-        _0016: {RP: {}},
-        # _10E0: {},  # 13: will not RP/10E0 # TODO: how to indicate that fact here
-        _1100: {I_: {}, RP: {}},
-        _1FC9: {RP: {}, W_: {}},
-        _3B00: {I_: {}},
-        _3EF0: {I_: {}},
-        # RP: {},  # RQ --- 01:145038 13:237335 --:------ 3EF0 001 00
-        _3EF1: {RP: {}},
-    },
-    "17": {
-        _0002: {I_: {}},
-        _1FC9: {I_: {}},
-    },  # i.e. HB85 (ext. temperature/luminosity(lux)), HB95 (+ wind speed)
-    "18": {
-        _3220: {RQ: {}},
-    },
-    "20": {  # HVAC: ventilation unit, or switch/sensor?
-        _10E0: {I_: {}, RP: {}},
-        _12A0: {RP: {}},
-        _22F1: {I_: {}},
-        _22F3: {I_: {}},
-        _3120: {RP: {}},
-        _31D9: {I_: {}, RP: {}},
-        _31DA: {I_: {}},
-    },  # e.g. https://www.ithodaalderop.nl/nl-NL/professional/product/545-5036
+    "18": {  # HGI80: RF to (USB) serial gateway interface
+        _PUZZ: {I_: {}, RQ: {}, W_: {}},
+    },  # HGI80s can do what they like
     "23": {
         _0009: {I_: {}},
         _1090: {RP: {}},
@@ -728,11 +859,7 @@ RAMSES_DEVICES = {
         _3B00: {I_: {}},
         _3EF1: {RP: {}},
     },
-    "29": {  # HVAC: Orcon MVS-15RP / MVS-15LF (vent mech. control)
-        _10E0: {I_: {}},  # VMC-15RP01 / VMN-15LF01
-        _31D9: {I_: {}},
-    },  # e.g. https://www.orcon.nl/blueline-mvs-15rp-2/
-    "30": {  # e.g. RFG100 (and others)
+    "30": {  # RFG100: RF to Internet gateway (and others)
         # RFG:185469 - Honeywell RFG100
         _0002: {RQ: {}},
         _0004: {I_: {}, RQ: {}},
@@ -764,19 +891,6 @@ RAMSES_DEVICES = {
         _31D9: {I_: {}},
         _31DA: {I_: {}, RP: {}},
     },
-    "31": {
-        _0008: {I_: {}},
-        _10E0: {I_: {}},
-        _3EF1: {RQ: {}, RP: {}},
-    },
-    "32": {  # HVAC: switch/sensor?
-        _1060: {I_: {}},
-        _10E0: {I_: {}, RP: {}},
-        _12A0: {I_: {}},
-        _22F1: {I_: {}},
-        _31DA: {RQ: {}},
-        _31E0: {I_: {}},
-    },
     "34": {  # e.g. TR87RF: Single (round) Zone Thermostat
         _0005: {I_: {}},
         _0008: {I_: {}},
@@ -796,6 +910,35 @@ RAMSES_DEVICES = {
         _3EF0: {RQ: {}},  # when bound direct to a 13:
         _3EF1: {RQ: {}},  # when bound direct to a 13:
     },
+}
+RAMSES_DEVICES["00"] = RAMSES_DEVICES_CLASS["TRV"]  # HR80
+RAMSES_DEVICES["21"] = RAMSES_DEVICES["34"]  # T87RF1003
+RAMSES_DEVICES["22"] = RAMSES_DEVICES["12"]  # DTS92
+
+HVAC_DEVICES = {
+    "20": {  # HVAC: ventilation unit, or switch/sensor?
+        _10E0: {I_: {}, RP: {}},
+        _12A0: {RP: {}},
+        _22F1: {I_: {}},
+        _22F3: {I_: {}},
+        _3120: {RP: {}},
+        _31D9: {I_: {}, RP: {}},
+        _31DA: {I_: {}},
+    },  # e.g. https://www.ithodaalderop.nl/nl-NL/professional/product/545-5036
+    "29": {  # HVAC: Orcon MVS-15RP / MVS-15LF (vent mech. control)
+        _10E0: {I_: {}},  # VMC-15RP01 / VMN-15LF01
+        _31D9: {I_: {}},
+    },  # e.g. https://www.orcon.nl/blueline-mvs-15rp-2/
+    "32": {  # HVAC: switch/sensor?
+        _042F: {I_: {}},
+        _1060: {I_: {}},
+        _10E0: {I_: {}, RP: {}},
+        _1298: {I_: {}},
+        _12A0: {I_: {}},
+        _22F1: {I_: {}},
+        _31DA: {RQ: {}},
+        _31E0: {I_: {}},
+    },
     "37": {  # HVAC: ventilation unit
         _10E0: {I_: {}, RP: {}},
         _1298: {I_: {}},
@@ -809,9 +952,12 @@ RAMSES_DEVICES = {
         _22F3: {I_: {}},
     },  # https://www.ithodaalderop.nl/nl-NL/professional/product/536-0124
 }
-RAMSES_DEVICES["00"] = RAMSES_DEVICES["04"]  # HR80
-RAMSES_DEVICES["21"] = RAMSES_DEVICES["34"]  # T87RF1003
-RAMSES_DEVICES["22"] = RAMSES_DEVICES["12"]  # DTS92
+
+RAMSES_DEVICES = {**HVAC_DEVICES, **RAMSES_DEVICES}
+for k, v in DEVICE_TYPE_BY_CLASS.items():
+    if k in RAMSES_DEVICES_CLASS:
+        RAMSES_DEVICES[v] = RAMSES_DEVICES_CLASS[k]
+
 
 ##############
 # RAMSES_ZONES (WIP)
@@ -854,3 +1000,24 @@ RAMSES_ZONES = {
 RAMSES_ZONES_ALL = RAMSES_ZONES.pop("ALL")
 RAMSES_ZONES_DHW = RAMSES_ZONES["DHW"]
 [RAMSES_ZONES[k].update(RAMSES_ZONES_ALL) for k in RAMSES_ZONES if k != "DHW"]
+
+
+# _DEVICE_SIGNATURES = {
+#     "CTL": ((_0100, I_), (_0100, RP), (_0404, I_), (_0404, RP), (_0418, I_), (_0418, RP), ),
+#     "UFC": (),
+#     "DHW": ((_1260, I_)),
+#     "BDR": (),
+#     "OTB": ((_3220, RP)),
+#     "EXT": (),  # OUT?
+#     "RFG": (),
+#     "VNS": (),
+#     "VNT": ((_31D9, I_), (_31D9, RP), (_31DA, I_), (_31DA, RP)),
+# }
+
+# DEVICE_SIGNATURES = {}
+# for device, signatures in _DEVICE_SIGNATURES.items():
+#     for pair in signatures:
+#         if (code := pair[1]) in DEVICE_SIGNATURES:
+#             raise RuntimeError
+#         else:
+#             DEVICE_SIGNATURES[device] = {code: pair[0]}
