@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-"""RAMSES RF - RAMSES-II compatble Packet processor.
+"""RAMSES RF - RAMSES-II compatible Packet processor.
 
 Operates at the pkt layer of: app - msg - pkt - h/w
 """
@@ -114,16 +114,21 @@ def _normalise(pkt_line: str) -> str:
     return pkt_line.strip()
 
 
-def _regex_hack(pkt_line: str, regex_filters) -> str:
+def _regex_hack(pkt_line: str, regex_filters: dict) -> str:
     """Perform any packet hacks, as configured."""
 
+    result = pkt_line
+
     for k, v in regex_filters.items():
         try:
-            pkt_line = re.sub(k, v, pkt_line)
+            result = re.sub(k, v, result)
         except re.error as exc:
             _LOGGER.warning(f"{pkt_line} < issue with regex ({k}, {v}): {exc}")
 
-    return pkt_line
+    if result != pkt_line:
+        _LOGGER.warning(f"{pkt_line} < Changed by use_regex to: {result}")
+
+    return result
 
 
 class SerTransportRead(asyncio.ReadTransport):
@@ -607,7 +612,7 @@ class PacketProtocolRead(PacketProtocolBase):
                 self._hgi80[DEVICE_ID] = pkt.src.id
 
             elif self._hgi80[DEVICE_ID] != pkt.src.id:
-                (_LOGGER.debug if pkt.src.id in self._unwanted else _LOGGER.warning)(
+                _LOGGER.debug(
                     f"{pkt} < There appears to be more than one HGI80-compatible device"
                     f" (active gateway: {self._hgi80[DEVICE_ID]}), this is unsupported"
                 )
@@ -836,6 +841,17 @@ def create_pkt_stack(
      - MsgTransport.write (pkt_dispatcher) to (pkt_protocol) PktProtocol.send_data
     """
 
+    def issue_warning():
+        _LOGGER.warning(
+            "Windows "
+            if os.name == "nt"
+            else "This type of serial interface "
+            "is not fully supported by this library: "
+            "please don't report any Transport/Protocol errors/warnings, "
+            "unless they are reproducable with a standard configuration "
+            "(e.g. linux with a local serial port)"
+        )
+
     def _protocol_factory():
         if packet_log or packet_dict is not None:
             return create_protocol_factory(PacketProtocolRead, gwy, pkt_callback)()
@@ -877,6 +893,7 @@ def create_pkt_stack(
             os.name == "nt",
         )
     ):
+        issue_warning()
         pkt_transport = SerTransportPoll(gwy._loop, pkt_protocol, ser_instance)
     else:
         pkt_transport = SerTransportAsync(gwy._loop, pkt_protocol, ser_instance)
