@@ -7,13 +7,23 @@ import logging
 from asyncio import Task
 from datetime import datetime as dt
 from datetime import timedelta as td
-from inspect import getmembers, isclass
-from sys import modules
 from threading import Lock
 from types import SimpleNamespace
 from typing import List, Optional
 
-from .const import Discover
+from .const import (
+    _000C_DEVICE,
+    _0005_ZONE,
+    ATTR_DATETIME,
+    ATTR_DEVICES,
+    ATTR_HEAT_DEMAND,
+    ATTR_LANGUAGE,
+    ATTR_SYSTEM_MODE,
+    SYSTEM_MODE,
+    Discover,
+    SystemType,
+    __dev_mode__,
+)
 from .devices import (
     BdrSwitch,
     Device,
@@ -22,39 +32,31 @@ from .devices import (
     Temperature,
     UfhController,
 )
-from .entities import Entity, discover_decorator
-from .protocol import Command, FaultLog, Priority
-from .protocol.const import (
-    _000C_DEVICE,
-    _0005_ZONE,
-    ATTR_CONTROLLER,
-    ATTR_DATETIME,
-    ATTR_DEVICES,
-    ATTR_DHW_SENSOR,
-    ATTR_DHW_VALVE,
-    ATTR_DHW_VALVE_HTG,
-    ATTR_HEAT_DEMAND,
-    ATTR_LANGUAGE,
-    ATTR_SYSTEM,
-    ATTR_SYSTEM_MODE,
-    ATTR_ZONE_SENSOR,
-    SYSTEM_MODE,
-    ZONE_TYPE_SLUGS,
-    SystemType,
+from .entities import Entity, class_by_attr, discover_decorator
+from .protocol import (
+    Command,
+    CorruptStateError,
+    ExpiredCallbackError,
+    FaultLog,
+    Priority,
 )
-from .protocol.exceptions import CorruptStateError, ExpiredCallbackError
 from .protocol.transport import PacketProtocolPort
 from .schema import (
-    ATTR_DHW_SYSTEM,
-    ATTR_HTG_CONTROL,
-    ATTR_HTG_SYSTEM,
-    ATTR_ORPHANS,
-    ATTR_UFH_SYSTEM,
-    ATTR_ZONES,
+    SZ_CONTROLLER,
+    SZ_DHW_SENSOR,
+    SZ_DHW_SYSTEM,
+    SZ_DHW_VALVE,
+    SZ_DHW_VALVE_HTG,
+    SZ_HTG_CONTROL,
+    SZ_HTG_SYSTEM,
+    SZ_ORPHANS,
+    SZ_UFH_SYSTEM,
+    SZ_ZONE_SENSOR,
+    SZ_ZONES,
 )
 from .zones import DhwZone, Zone, create_zone
 
-from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
     _0001,
     _0002,
@@ -116,7 +118,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _PUZZ,
 )
 
-DEV_MODE = __dev_mode__ and False
+DEV_MODE = __dev_mode__
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -294,11 +296,11 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
             return
         if self._htg_control is not None:
             raise CorruptStateError(
-                f"{self} changed {ATTR_HTG_CONTROL}: {self._htg_control} to {device}"
+                f"{self} changed {SZ_HTG_CONTROL}: {self._htg_control} to {device}"
             )
 
         if not isinstance(device, (BdrSwitch, OtbGateway)):
-            raise TypeError(f"{self}: {ATTR_HTG_CONTROL} can't be {device}")
+            raise TypeError(f"{self}: {SZ_HTG_CONTROL} can't be {device}")
 
         self._htg_control = device
         device._set_parent(self, domain="FC")  # TODO: _set_domain()
@@ -324,14 +326,14 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
     def schema(self) -> dict:
         """Return the system's schema."""
 
-        schema = {ATTR_HTG_SYSTEM: {}}
-        # hema = {ATTR_CONTROLLER: self._ctl.id, ATTR_HTG_SYSTEM: {}}
+        schema = {SZ_HTG_SYSTEM: {}}
+        # hema = {SZ_CONTROLLER: self._ctl.id, SZ_HTG_SYSTEM: {}}
 
-        schema[ATTR_HTG_SYSTEM][ATTR_HTG_CONTROL] = (
+        schema[SZ_HTG_SYSTEM][SZ_HTG_CONTROL] = (
             self.heating_control.id if self.heating_control else None
         )
 
-        schema[ATTR_ORPHANS] = sorted(
+        schema[SZ_ORPHANS] = sorted(
             [
                 d.id
                 for d in self._ctl.devices
@@ -346,27 +348,27 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
         """Return the global schema."""
 
         schema = self.schema
-        result = {ATTR_CONTROLLER: self.id}
+        result = {SZ_CONTROLLER: self.id}
 
         try:
-            if schema[ATTR_SYSTEM][ATTR_HTG_CONTROL][:2] == "10":  # DEX
-                result[ATTR_SYSTEM] = {
-                    ATTR_HTG_CONTROL: schema[ATTR_SYSTEM][ATTR_HTG_CONTROL]
+            if schema[SZ_HTG_SYSTEM][SZ_HTG_CONTROL][:2] == "10":  # DEX
+                result[SZ_HTG_SYSTEM] = {
+                    SZ_HTG_CONTROL: schema[SZ_HTG_SYSTEM][SZ_HTG_CONTROL]
                 }
         except (IndexError, TypeError):
-            result[ATTR_SYSTEM] = {ATTR_HTG_CONTROL: None}
+            result[SZ_HTG_SYSTEM] = {SZ_HTG_CONTROL: None}
 
         zones = {}
-        for idx, zone in schema[ATTR_ZONES].items():
+        for idx, zone in schema[SZ_ZONES].items():
             _zone = {}
-            if zone[ATTR_ZONE_SENSOR] and zone[ATTR_ZONE_SENSOR][:2] == "01":  # DEX
-                _zone = {ATTR_ZONE_SENSOR: zone[ATTR_ZONE_SENSOR]}
+            if zone[SZ_ZONE_SENSOR] and zone[SZ_ZONE_SENSOR][:2] == "01":  # DEX
+                _zone = {SZ_ZONE_SENSOR: zone[SZ_ZONE_SENSOR]}
             if devices := [d for d in zone[ATTR_DEVICES] if d[:2] == "00"]:  # DEX
                 _zone.update({ATTR_DEVICES: devices})
             if _zone:
                 zones[idx] = _zone
         if zones:
-            result[ATTR_ZONES] = zones
+            result[SZ_ZONES] = zones
 
         return result
 
@@ -374,16 +376,16 @@ class SystemBase(Entity):  # 3B00 (multi-relay)
     def params(self) -> dict:
         """Return the system's configuration."""
 
-        params = {ATTR_HTG_SYSTEM: {}}
-        params[ATTR_HTG_SYSTEM]["tpi_params"] = self._msg_value(_1100)
+        params = {SZ_HTG_SYSTEM: {}}
+        params[SZ_HTG_SYSTEM]["tpi_params"] = self._msg_value(_1100)
         return params
 
     @property
     def status(self) -> dict:
         """Return the system's current state."""
 
-        status = {ATTR_HTG_SYSTEM: {}}
-        status[ATTR_HTG_SYSTEM]["heat_demand"] = self.heat_demand
+        status = {SZ_HTG_SYSTEM: {}}
+        status[SZ_HTG_SYSTEM]["heat_demand"] = self.heat_demand
 
         status[ATTR_DEVICES] = {d.id: d.status for d in sorted(self._ctl.devices)}
 
@@ -422,33 +424,36 @@ class MultiZone:  # 0005 (+/- 000C?)
                     self._make_cmd(_0005, payload=f"00{zone_type}")
 
     def _handle_msg(self, msg) -> None:
+        def handle_msg_by_zone_idx(zone_idx: str, msg):
+            if zone_idx is None:
+                pass
+            elif zone := self.zone_by_idx.get(zone_idx):
+                zone._handle_msg(msg)
+            # elif self._gwy.config.enable_eavesdrop:
+            #     self._get_zone(zone_idx)._handle_msg(msg)
+
         super()._handle_msg(msg)
 
         # TODO: a I/0005 may have changed zones & may need a restart (del) or not (add)
         if msg.code == _0005:  # RP, and also I
             if msg.payload.get("_device_class") in self.ZONE_TYPES:
                 [
-                    self._get_zone(f"{idx:02X}", **msg.payload)
+                    self._get_zone(f"{idx:02X}", msg=msg)
                     for idx, flag in enumerate(msg.payload["zone_mask"])
                     if flag == 1
                 ]
             return
 
-        # Route any other messages to their zones, incl. 000C
-        try:
-            if isinstance(msg.payload, dict):  # incl. 000C
-                if "zone_idx" in msg.payload:
-                    self._evo.zone_by_idx[msg.payload["zone_idx"]]._handle_msg(msg)
-                # elif msg.payload.get("domain_id") == "FA":  # DHW
+        # NOTE: Route all messages to their zones, incl. 000C, others
+        if isinstance(msg.payload, dict):
+            if zone_idx := msg.payload.get("zone_idx"):
+                handle_msg_by_zone_idx(zone_idx, msg)
+            # TODO: elif msg.payload.get("domain_id") == "FA":  # DHW
 
-            elif isinstance(msg.payload, list):
-                [
-                    self._evo.zone_by_idx[z["zone_idx"]]._handle_msg(msg)
-                    for z in msg.payload
-                    if "zone_idx" in msg.payload[0]
-                ]
-        except KeyError:  # FIXME: this shouldn't happen
-            pass  # if self._gwy.config.enable_eavesdrop:
+        elif isinstance(msg.payload, list):
+            if msg.payload[0].get("zone_idx"):
+                [handle_msg_by_zone_idx(z.get("zone_idx"), msg) for z in msg.payload]
+            # TODO: elif msg.payload.get("domain_id") == "FA":  # DHW
 
         if self._gwy.config.enable_eavesdrop and not all(z.sensor for z in self.zones):
             self._eavesdrop_zone_sensors(msg)
@@ -595,21 +600,13 @@ class MultiZone:  # 0005 (+/- 000C?)
 
         _LOGGER.debug("System state (finally): %s", self.schema)
 
-    def _get_zone(self, zone_idx, **kwargs) -> Zone:
-        """Return a heating zone (will create/update it if required)."""
+    def _get_zone(self, zone_idx, msg=None, **kwargs) -> Zone:
+        """Return a heating zone (will create it if required)."""
 
-        # NOTE: kwargs not passed, only so discovery is as eavesdropping
-        zone = self.zone_by_idx.get(zone_idx) or create_zone(self, zone_idx)
+        zone = self.zone_by_idx.get(zone_idx) or create_zone(self, zone_idx, **kwargs)
 
-        if kwargs.get("_device_class"):
-            zone._set_zone_type(ZONE_TYPE_SLUGS.get(kwargs["zone_type"]))
-
-        if kwargs.get("sensor"):
-            zone._set_sensor(kwargs["sensor"])
-
-        if kwargs.get("actuators"):  # TODO: check not an address before implementing
-            for device in [d for d in kwargs["actuators"] if d not in zone.devices]:
-                device._set_parent(zone)
+        if msg and (zone_type := msg.payload.get("zone_type")):
+            zone._set_zone_type(zone_type)
 
         return zone
 
@@ -617,21 +614,21 @@ class MultiZone:  # 0005 (+/- 000C?)
     def schema(self) -> dict:
         return {
             **super().schema,
-            ATTR_ZONES: {z.idx: z.schema for z in sorted(self.zones)},
+            SZ_ZONES: {z.idx: z.schema for z in sorted(self.zones)},
         }
 
     @property
     def params(self) -> dict:
         return {
             **super().params,
-            ATTR_ZONES: {z.idx: z.params for z in sorted(self.zones)},
+            SZ_ZONES: {z.idx: z.params for z in sorted(self.zones)},
         }
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            ATTR_ZONES: {z.idx: z.status for z in sorted(self.zones)},
+            SZ_ZONES: {z.idx: z.status for z in sorted(self.zones)},
         }
 
 
@@ -657,8 +654,8 @@ class ScheduleSync:  # 0006
                 or dt.now() - self._active_0006.dtm > td(minutes=3)
             ):
                 self._active_0006 = msg  # TODO: what happens if the following fails?
-                if not self._gwy.config.disable_sending:
-                    self._get_schedules()
+                # if not self._gwy.config.disable_sending:  # TODO
+                #     self._get_schedules()
 
     def _get_schedules(self) -> None:
         if self._gwy.config.disable_sending:
@@ -699,7 +696,7 @@ class Language:  # 0100
     @property
     def params(self) -> dict:
         params = super().params
-        params[ATTR_HTG_SYSTEM][ATTR_LANGUAGE] = self.language
+        params[SZ_HTG_SYSTEM][ATTR_LANGUAGE] = self.language
         return params
 
 
@@ -780,10 +777,11 @@ class StoredHw:  # 10A0, 1260, 1F41
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
+        # TODO: a I/0005 may have changed zones & may need a restart (del) or not (add)
         if msg.code == _000C:
             if (
                 msg.payload["device_class"]
-                in (ATTR_DHW_SENSOR, ATTR_DHW_VALVE, ATTR_DHW_VALVE_HTG)
+                in (SZ_DHW_SENSOR, SZ_DHW_VALVE, SZ_DHW_VALVE_HTG)
                 and msg.payload["devices"]
             ):
                 self._get_dhw()._handle_msg(msg)
@@ -841,16 +839,18 @@ class StoredHw:  # 10A0, 1260, 1F41
     def _get_dhw(self, **kwargs) -> DhwZone:
         """Return the DHW zone (will create/update it if required)."""
 
+        # return self.dhw or create_zone(self, zone_idx="HW")
+
         dhw = self.dhw or create_zone(self, zone_idx="HW")
 
-        if kwargs.get(ATTR_DHW_SENSOR):
-            dhw._set_sensor(kwargs[ATTR_DHW_SENSOR])
+        if kwargs.get(SZ_DHW_SENSOR):
+            dhw._set_sensor(kwargs[SZ_DHW_SENSOR])
 
-        if kwargs.get(ATTR_DHW_VALVE):
-            dhw._set_dhw_valve(kwargs[ATTR_DHW_VALVE])
+        if kwargs.get(SZ_DHW_VALVE):
+            dhw._set_dhw_valve(kwargs[SZ_DHW_VALVE])
 
-        if kwargs.get(ATTR_DHW_VALVE_HTG):
-            dhw._set_htg_valve(kwargs[ATTR_DHW_VALVE_HTG])
+        if kwargs.get(SZ_DHW_VALVE_HTG):
+            dhw._set_htg_valve(kwargs[SZ_DHW_VALVE_HTG])
 
         self._dhw = dhw
         return dhw
@@ -892,21 +892,21 @@ class StoredHw:  # 10A0, 1260, 1F41
     def schema(self) -> dict:
         return {
             **super().schema,
-            ATTR_DHW_SYSTEM: self._dhw.schema if self._dhw else {},
+            SZ_DHW_SYSTEM: self._dhw.schema if self._dhw else {},
         }
 
     @property
     def params(self) -> dict:
         return {
             **super().params,
-            ATTR_DHW_SYSTEM: self._dhw.params if self._dhw else {},
+            SZ_DHW_SYSTEM: self._dhw.params if self._dhw else {},
         }
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            ATTR_DHW_SYSTEM: self._dhw.status if self._dhw else {},
+            SZ_DHW_SYSTEM: self._dhw.status if self._dhw else {},
         }
 
 
@@ -938,7 +938,7 @@ class SysMode:  # 2E04
     @property
     def params(self) -> dict:
         params = super().params
-        params[ATTR_HTG_SYSTEM][ATTR_SYSTEM_MODE] = self.system_mode
+        params[SZ_HTG_SYSTEM][ATTR_SYSTEM_MODE] = self.system_mode
         return params
 
 
@@ -985,21 +985,21 @@ class UfHeating:
     def schema(self) -> dict:
         return {
             **super().schema,
-            ATTR_UFH_SYSTEM: {d.id: d.schema for d in self._ufh_ctls()},
+            SZ_UFH_SYSTEM: {d.id: d.schema for d in self._ufh_ctls()},
         }
 
     @property
     def params(self) -> dict:
         return {
             **super().params,
-            ATTR_UFH_SYSTEM: {d.id: d.params for d in self._ufh_ctls()},
+            SZ_UFH_SYSTEM: {d.id: d.params for d in self._ufh_ctls()},
         }
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            ATTR_UFH_SYSTEM: {d.id: d.status for d in self._ufh_ctls()},
+            SZ_UFH_SYSTEM: {d.id: d.status for d in self._ufh_ctls()},
         }
 
 
@@ -1056,11 +1056,11 @@ class System(StoredHw, Datetime, Logbook, SystemBase):
         """Return the system's current state."""
 
         status = super().status
-        # assert ATTR_HTG_SYSTEM in status  # TODO: removeme
+        # assert SZ_HTG_SYSTEM in status  # TODO: removeme
 
-        status[ATTR_HTG_SYSTEM]["heat_demands"] = self.heat_demands
-        status[ATTR_HTG_SYSTEM]["relay_demands"] = self.relay_demands
-        status[ATTR_HTG_SYSTEM]["relay_failsafes"] = self.relay_failsafes
+        status[SZ_HTG_SYSTEM]["heat_demands"] = self.heat_demands
+        status[SZ_HTG_SYSTEM]["relay_demands"] = self.relay_demands
+        status[SZ_HTG_SYSTEM]["relay_failsafes"] = self.relay_failsafes
 
         return status
 
@@ -1171,23 +1171,19 @@ _SYS_CLASS_BY_TYPE = {
 }
 
 
-_PROFILE = "_PROFILE"
-SYSTEM_BY_PROFILE = {
-    getattr(c[1], _PROFILE): c[1]
-    for c in getmembers(
-        modules[__name__],
-        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, _PROFILE),
-    )
-}  # e.g. "evohome": Evohome
+_SYS_CLASS_BY_PROFILE = class_by_attr(__name__, "_PROFILE")  # e.g. "evohome": Evohome
 
 
 def create_system(gwy, ctl, profile=None, **kwargs) -> System:
     """Create a system, and optionally perform discovery & start polling."""
 
+    #
+
     if profile is None:  # TODO: messy
         profile = (
             SYSTEM_PROFILE.PRG if isinstance(ctl, Programmer) else SYSTEM_PROFILE.EVO
         )
+        #
 
     system = _SYS_CLASS.get(profile, System)(gwy, ctl, **kwargs)
 
