@@ -7,12 +7,25 @@ import logging
 from asyncio import Task
 from datetime import datetime as dt
 from datetime import timedelta as td
-from inspect import getmembers, isclass
-from sys import modules
 from types import SimpleNamespace
 from typing import Optional
 
-from .const import Discover, __dev_mode__
+from .const import (
+    _000C_DEVICE,
+    ATTR_DEVICES,
+    ATTR_HEAT_DEMAND,
+    ATTR_NAME,
+    ATTR_RELAY_DEMAND,
+    ATTR_RELAY_FAILSAFE,
+    ATTR_SETPOINT,
+    ATTR_TEMP,
+    ATTR_WINDOW_OPEN,
+    ZONE_MODE,
+    ZONE_TYPE_MAP,
+    ZONE_TYPE_SLUGS,
+    Discover,
+    __dev_mode__,
+)
 from .devices import (
     BdrSwitch,
     Controller,
@@ -22,31 +35,20 @@ from .devices import (
     TrvActuator,
     UfhController,
 )
-from .entities import Entity, discover_decorator
-from .protocol import CODE_API_MAP, Command, Schedule
-from .protocol.const import (
-    _000C_DEVICE,
-    ATTR_DEVICES,
-    ATTR_DHW_SENSOR,
-    ATTR_DHW_VALVE,
-    ATTR_DHW_VALVE_HTG,
-    ATTR_HEAT_DEMAND,
-    ATTR_NAME,
-    ATTR_RELAY_DEMAND,
-    ATTR_RELAY_FAILSAFE,
-    ATTR_SETPOINT,
-    ATTR_TEMP,
-    ATTR_WINDOW_OPEN,
-    ATTR_ZONE_SENSOR,
-    ATTR_ZONE_TYPE,
-    ZONE_MODE,
-    ZONE_TYPE_MAP,
-    ZONE_TYPE_SLUGS,
-)
-from .protocol.exceptions import CorruptStateError
+from .entities import Entity, class_by_attr, discover_decorator
+from .protocol import CODE_API_MAP, Command, CorruptStateError, Schedule
 from .protocol.transport import PacketProtocolPort
+from .schema import (
+    SZ_DEVICE_ID,
+    SZ_DHW_SENSOR,
+    SZ_DHW_VALVE,
+    SZ_DHW_VALVE_HTG,
+    SZ_ZONE_SENSOR,
+    SZ_ZONE_TYPE,
+)
 
 # from .ramses import RAMSES_ZONES, RAMSES_ZONES_ALL
+
 from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
     _0001,
@@ -109,14 +111,14 @@ from .protocol import (  # noqa: F401, isort: skip
     _PUZZ,
 )
 
-DEV_MODE = __dev_mode__ and False
+DEV_MODE = __dev_mode__
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-ZONE_TYPE = SimpleNamespace(
+ZON_KLASS = SimpleNamespace(
     DHW="DHW",  # Stored HW (not a zone)
     ELE="ELE",  # Electric
     MIX="MIX",  # Mix valve
@@ -129,7 +131,7 @@ ZONE_TYPE = SimpleNamespace(
 class ZoneBase(Entity):
     """The Zone/DHW base class."""
 
-    # _TYPE = None  # NOTE: this would cause problems
+    # _klass = None  # NOTE: this would cause problems
 
     def __init__(self, evo, zone_idx) -> None:
         _LOGGER.debug("Creating a Zone: %s_%s (%s)", evo, zone_idx, self.__class__)
@@ -192,7 +194,7 @@ class ZoneBase(Entity):
     @property
     def heating_type(self) -> Optional[str]:
         """Return the type of the zone/DHW (e.g. electric_zone, stored_dhw)."""
-        return self._TYPE
+        return self._klass
 
 
 class ZoneSchedule:  # 0404  # TODO: add for DHW
@@ -267,7 +269,7 @@ class RelayDemand:  # 0008
 class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
     """The DHW class."""
 
-    _TYPE = ZONE_TYPE.DHW
+    _klass = ZON_KLASS.DHW
 
     def __init__(
         self, ctl, zone_idx="HW", sensor=None, dhw_valve=None, htg_valve=None
@@ -275,14 +277,14 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         super().__init__(ctl, zone_idx)
 
         ctl._set_dhw(self)
-        # if profile == ZONE_TYPE.DHW and evo.dhw is None:
+        # if profile == ZON_KLASS.DHW and evo.dhw is None:
         #     evo.dhw = zone
 
         self._dhw_sensor = None
         self._dhw_valve = None
         self._htg_valve = None
 
-        self._zone_type = ZONE_TYPE.DHW
+        self._zone_type = ZON_KLASS.DHW
 
         if sensor:
             self._set_sensor(sensor)
@@ -332,11 +334,24 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         assert msg.src is self._ctl, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
+        # if msg.code == _000C:
+        #     if kwargs.get(ATTR_DHW_SENSOR):
+        #         dhw._set_sensor(kwargs[ATTR_DHW_SENSOR])
+
+        #     if kwargs.get(SZ_DHW_VALVE):
+        #         dhw._set_dhw_valve(kwargs[SZ_DHW_VALVE])
+
+        #     if kwargs.get(SZ_DHW_VALVE_HTG):
+        #         dhw._set_htg_valve(kwargs[SZ_DHW_VALVE_HTG])
+
+        #     self._dhw = dhw
+        #     return dhw
+
         if msg.code == _000C and msg.payload["devices"]:
             LOOKUP = {
-                ATTR_DHW_SENSOR: self._set_sensor,
-                ATTR_DHW_VALVE: self._set_dhw_valve,
-                ATTR_DHW_VALVE_HTG: self._set_htg_valve,
+                SZ_DHW_SENSOR: self._set_sensor,
+                SZ_DHW_VALVE: self._set_dhw_valve,
+                SZ_DHW_VALVE_HTG: self._set_htg_valve,
             }
             devices = [
                 # self._gwy._get_device(d, ctl_id=msg.src.id, domain_id=...)
@@ -367,7 +382,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         # 07:38:39.140 062 RP --- 01:102458 07:030741 --:------ 10A0 006 0018380003E8
 
         self._dhw_sensor = self._set_dhw_device(
-            device, self._dhw_sensor, ATTR_ZONE_SENSOR, DhwSensor, "FA"
+            device, self._dhw_sensor, SZ_ZONE_SENSOR, DhwSensor, "FA"
         )
 
     @property
@@ -387,7 +402,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         """  # noqa: D402
 
         self._dhw_valve = self._set_dhw_device(
-            device, self._dhw_valve, ATTR_DHW_VALVE, BdrSwitch, "FA"
+            device, self._dhw_valve, SZ_DHW_VALVE, BdrSwitch, "FA"
         )
 
     @property
@@ -398,7 +413,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         """Set the heating valve relay for this DHW system (13: only)."""
 
         self._htg_valve = self._set_dhw_device(
-            device, self._htg_valve, ATTR_DHW_VALVE_HTG, BdrSwitch, "F9"
+            device, self._htg_valve, SZ_DHW_VALVE_HTG, BdrSwitch, "F9"
         )
 
     @property
@@ -479,9 +494,9 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
     def schema(self) -> dict:
         """Return the DHW's schema (devices)."""
         return {
-            ATTR_DHW_SENSOR: self.sensor.id if self.sensor else None,
-            ATTR_DHW_VALVE: self.hotwater_valve.id if self.hotwater_valve else None,
-            ATTR_DHW_VALVE_HTG: self.heating_valve.id if self.heating_valve else None,
+            SZ_DHW_SENSOR: self.sensor.id if self.sensor else None,
+            SZ_DHW_VALVE: self.hotwater_valve.id if self.hotwater_valve else None,
+            SZ_DHW_VALVE_HTG: self.heating_valve.id if self.heating_valve else None,
         }
 
     @property
@@ -498,7 +513,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
 class Zone(ZoneSchedule, ZoneBase):
     """The Zone class for all zone types (but not DHW)."""
 
-    _TYPE = None  # Unknown
+    _klass = None  # Unknown
 
     def __init__(self, evo, zone_idx, sensor=None, actuators=None) -> None:
         """Create a zone.
@@ -577,6 +592,20 @@ class Zone(ZoneSchedule, ZoneBase):
 
         super()._handle_msg(msg)
 
+        if msg.code == _000C:
+            # self._set_zone_type(msg.payload["zone_type"])
+
+            if msg.payload.get("sensor"):
+                self._set_sensor(msg.payload["sensor"])
+
+            if msg.payload.get("actuators"):
+                # TODO: confirm is/isn't an address before implementing
+                [
+                    d._set_parent(self)
+                    for d in msg.payload["actuators"]
+                    if d not in self.devices
+                ]
+
         if msg.code == _000C and msg.payload["devices"]:
 
             # TODO: testing this concept, hoping to learn device_id of UFC
@@ -650,12 +679,12 @@ class Zone(ZoneSchedule, ZoneBase):
             return
         if self._sensor is not None:
             raise CorruptStateError(
-                f"{self} changed {ATTR_ZONE_SENSOR}: {self._sensor} to {device}"
+                f"{self} changed {SZ_ZONE_SENSOR}: {self._sensor} to {device}"
             )
 
         if not isinstance(device, (Controller, Temperature)):
             # TODO: or not hasattr(device, "temperature")
-            raise TypeError(f"{self}: {ATTR_ZONE_SENSOR} can't be: {device}")
+            raise TypeError(f"{self}: {SZ_ZONE_SENSOR} can't be: {device}")
 
         self._sensor = device
         device._set_parent(self)  # , domain=self.idx)
@@ -677,7 +706,7 @@ class Zone(ZoneSchedule, ZoneBase):
         """
 
         _type = ZONE_TYPE_SLUGS.get(zone_type, zone_type)
-        if _type not in ZONE_BY_TYPE:
+        if _type not in _ZON_CLASS_BY_KLASS:
             raise ValueError(f"Not a known zone_type: {zone_type}")
 
         if self._zone_type == _type:
@@ -690,8 +719,9 @@ class Zone(ZoneSchedule, ZoneBase):
             )
 
         self._zone_type = _type
-        self.__class__ = ZONE_BY_TYPE[_type]
+        self.__class__ = _ZON_CLASS_BY_KLASS[_type]
         self._discover(discover_flag=Discover.ALL)  # TODO: needs tidyup (ref #67)
+        _LOGGER.debug("Promoted a Zone: %s(%s)", self.id, self.__class__)
 
     @property
     def name(self) -> Optional[str]:  # 0004
@@ -806,16 +836,16 @@ class Zone(ZoneSchedule, ZoneBase):
             sensor_schema = self._sensor.id
         else:
             sensor_schema = {
-                "device_id": self._sensor.id,
+                SZ_DEVICE_ID: self._sensor.id,
                 "is_faked": self._sensor._fake_30C9,
             }
 
         return {
             "_name": self.name,
-            ATTR_ZONE_TYPE: self.heating_type,
-            ATTR_ZONE_SENSOR: sensor_schema,
-            ATTR_DEVICES: [d.id for d in self.devices],
+            SZ_ZONE_TYPE: self.heating_type,
+            SZ_ZONE_SENSOR: sensor_schema,
             "sensor_alt": self.sensor.id if self.sensor else None,
+            ATTR_DEVICES: [d.id for d in self.devices],
             "actuators": self.actuators,
         }
 
@@ -835,7 +865,7 @@ class Zone(ZoneSchedule, ZoneBase):
 class EleZone(RelayDemand, Zone):  # BDR91A/T  # TODO: 0008/0009/3150
     """For a small electric load controlled by a relay (never calls for heat)."""
 
-    _TYPE = ZONE_TYPE.ELE
+    _klass = ZON_KLASS.ELE
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
@@ -872,7 +902,7 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
     Note that HM80s are listen-only devices.
     """
 
-    _TYPE = ZONE_TYPE.MIX
+    _klass = ZON_KLASS.MIX
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
@@ -905,7 +935,7 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
 class RadZone(Zone):  # HR92/HR80
     """For radiators controlled by HR92s or HR80s (will also call for heat)."""
 
-    _TYPE = ZONE_TYPE.RAD
+    _klass = ZON_KLASS.RAD
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
@@ -924,7 +954,7 @@ class RadZone(Zone):  # HR92/HR80
 class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
     """For underfloor heating controlled by an HCE80/HCC80 (will also call for heat)."""
 
-    _TYPE = ZONE_TYPE.UFH
+    _klass = ZON_KLASS.UFH
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
@@ -959,7 +989,7 @@ class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
 class ValZone(EleZone):  # BDR91A/T
     """For a motorised valve controlled by a BDR91 (will also call for heat)."""
 
-    _TYPE = ZONE_TYPE.VAL
+    _klass = ZON_KLASS.VAL
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
@@ -980,23 +1010,21 @@ class ValZone(EleZone):  # BDR91A/T
         return self.relay_demand
 
 
-_TYPE = "_TYPE"
-ZONE_BY_TYPE = {
-    getattr(c[1], _TYPE): c[1]
-    for c in getmembers(
-        modules[__name__],
-        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, _TYPE),
-    )
-}  # e.g. "RAD": RadZone
+_ZON_CLASS_BY_KLASS = class_by_attr(__name__, "_klass")  # e.g. "RAD": RadZone
 
 
-def create_zone(evo, zone_idx, profile=None, **kwargs) -> Zone:
+def create_zone(evo, zone_idx: str, profile=None, **kwargs) -> Zone:
     """Create a zone, and optionally perform discovery & start polling."""
 
-    if profile is None:
-        profile = ZONE_TYPE.DHW if zone_idx == "HW" else None
+    #
 
-    zone = ZONE_BY_TYPE.get(profile, Zone)(evo, zone_idx, **kwargs)
+    if profile is None:
+        profile = ZON_KLASS.DHW if zone_idx == "HW" else None
+        #
+        #
+        #
+
+    zone = _ZON_CLASS_BY_KLASS.get(profile, Zone)(evo, zone_idx, **kwargs)
 
     if isinstance(evo._gwy.pkt_protocol, PacketProtocolPort):
         zone._start_discovery()
